#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests the plist parser."""

import unittest

from plaso.lib import errors
from plaso.parsers import plist
# Register all plugins.
from plaso.parsers import plist_plugins  # pylint: disable=unused-import

from tests.parsers import test_lib


class PlistParserTest(test_lib.ParserTestCase):
  """Tests the plist parser."""

  # pylint: disable=protected-access

  def testEnablePlugins(self):
    """Tests the EnablePlugins function."""
    parser = plist.PlistParser()

    number_of_plugins = len(parser._plugin_classes)

    parser.EnablePlugins([])
    self.assertEqual(len(parser._plugins_per_name), 0)

    parser.EnablePlugins(parser.ALL_PLUGINS)
    # Extract 1 for the default plugin.
    self.assertEqual(len(parser._plugins_per_name), number_of_plugins - 1)

    parser.EnablePlugins(['airport'])
    self.assertEqual(len(parser._plugins_per_name), 1)

  def testParse(self):
    """Tests the Parse function."""
    parser = plist.PlistParser()
    storage_writer = self._ParseFile(['plist_binary'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 12)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

  def testParseWithTruncatedFile(self):
    """Tests the Parse function on a truncated plist file."""
    parser = plist.PlistParser()

    with self.assertRaises(errors.WrongParser):
      self._ParseFile(['truncated.plist'], parser)

  def testParseWithXMLFileLeadingWhitespace(self):
    """Tests the Parse function on an XML file with leading whitespace."""
    parser = plist.PlistParser()
    storage_writer = self._ParseFile(['leading_whitespace.plist'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 4)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

  def testParseWithXMLFileInvalidDate(self):
    """Tests the Parse function on an XML file with an invalid date and time."""
    parser = plist.PlistParser()
    storage_writer = self._ParseFile(['com.apple.security.KCN.plist'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

  def testParseWithXMLFileExpatError(self):
    """Tests the Parse function on an XML file that causes an ExpatError."""
    parser = plist.PlistParser()

    with self.assertRaises(errors.WrongParser):
      self._ParseFile(['WMSDKNS.DTD'], parser)

  def testParseWithXMLFileBinASCIIError(self):
    """Tests the Parse function on an XML file that causes a binascii.Error."""
    parser = plist.PlistParser()

    with self.assertRaises(errors.WrongParser):
      self._ParseFile(['manageconsolidatedProviders.aspx.resx'], parser)

  def testParseWithXMLFileNoTopLevel(self):
    """Tests the Parse function on an XML file without top level."""
    parser = plist.PlistParser()

    with self.assertRaises(errors.WrongParser):
      test_path_segments = [
          'SettingsPane_{F8B5DB1C-D219-4bf9-A747-A1325024469B}'
          '.settingcontent-ms']
      self._ParseFile(test_path_segments, parser)

    # UTF-8 encoded XML file with byte-order-mark.
    with self.assertRaises(errors.WrongParser):
      self._ParseFile(['ReAgent.xml'], parser)

    # UTF-16 little-endian encoded XML file with byte-order-mark.
    with self.assertRaises(errors.WrongParser):
      self._ParseFile(['SampleMachineList.xml'], parser)

  def testParseWithXMLFileEncodingUnicode(self):
    """Tests the Parse function on an XML file with encoding Unicode."""
    parser = plist.PlistParser()

    with self.assertRaises(errors.WrongParser):
      self._ParseFile(['SAFStore.xml'], parser)

  def testParseWithEmptyBinaryPlistFile(self):
    """Tests the Parse function on an empty binary plist file."""
    parser = plist.PlistParser()
    storage_writer = self._ParseFile([
        'com.apple.networkextension.uuidcache.plist'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

  def testParseWithXMLPlistFileNoTopLevel(self):
    """Tests the Parse function on an XML plist file without top level."""
    parser = plist.PlistParser()
    storage_writer = self._ParseFile(['empty.plist'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

  def testParseWithXMLPlistFileEmptyTopLevel(self):
    """Tests the Parse function on an XML plist file with an empty top level."""
    parser = plist.PlistParser()
    storage_writer = self._ParseFile(['org.cups.printers.plist'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Plug-in to collect the Less Frequently Used (LFU) keys."""

from plaso.containers import events
from plaso.parsers import winreg_parser
from plaso.parsers.winreg_plugins import interface


class WindowsBootExecuteEventData(events.EventData):
  """Windows Boot Execute event data attribute container.

  Attributes:
    key_path (str): Windows Registry key path.
    last_written_time (dfdatetime.DateTimeValues): entry last written date and
        time.
    value (str): boot execute value, contains the value obtained from
        the BootExecute Registry value.
  """

  DATA_TYPE = 'windows:registry:boot_execute'

  def __init__(self):
    """Initializes event data."""
    super(WindowsBootExecuteEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.key_path = None
    self.last_written_time = None
    self.value = None


class WindowsBootVerificationEventData(events.EventData):
  """Windows Boot Verification event data attribute container.

  Attributes:
    image_path (str): location of the boot verification executable, contains
        the value obtained from the ImagePath Registry value.
    key_path (str): Windows Registry key path.
    last_written_time (dfdatetime.DateTimeValues): entry last written date and
        time.
  """

  DATA_TYPE = 'windows:registry:boot_verification'

  def __init__(self):
    """Initializes event data."""
    super(WindowsBootVerificationEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.image_path = None
    self.key_path = None
    self.last_written_time = None


class BootVerificationPlugin(interface.WindowsRegistryPlugin):
  """Plug-in to collect the Boot Verification Key."""

  NAME = 'windows_boot_verify'
  DATA_FORMAT = 'Windows boot verification Registry data'

  FILTERS = frozenset([
      interface.WindowsRegistryKeyPathFilter(
          'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\'
          'BootVerificationProgram')])

  def ExtractEvents(self, parser_mediator, registry_key, **kwargs):
    """Extracts events from a Windows Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
    """
    image_path = None
    registry_value = registry_key.GetValueByName('ImagePath')
    if registry_value:
      image_path = registry_value.GetDataAsObject()

    if image_path:
      event_data = WindowsBootVerificationEventData()
      event_data.key_path = registry_key.path
      event_data.image_path = image_path
      event_data.last_written_time = registry_key.last_written_time

      parser_mediator.ProduceEventData(event_data)

    self._ProduceDefaultWindowsRegistryEvent(
        parser_mediator, registry_key, names_to_skip=['ImagePath'])


class BootExecutePlugin(interface.WindowsRegistryPlugin):
  """Plug-in to collect the BootExecute Value from the Session Manager key."""

  NAME = 'windows_boot_execute'
  DATA_FORMAT = 'Boot Execution Registry data'

  FILTERS = frozenset([
      interface.WindowsRegistryKeyPathFilter(
          'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\'
          'Session Manager')])

  def ExtractEvents(self, parser_mediator, registry_key, **kwargs):
    """Extracts events from a Windows Registry key.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
    """
    registry_value = registry_key.GetValueByName('BootExecute')
    if registry_value:
      # MSDN: claims that the data type of this value is REG_BINARY
      # although REG_MULTI_SZ is known to be used.
      if registry_value.DataIsString():
        boot_execute = registry_value.GetDataAsObject()
      elif registry_value.DataIsMultiString():
        value_object = registry_value.GetDataAsObject()
        boot_execute = ', '.join(value_object or [])
      else:
        boot_execute = None
        error_string = (
            'Key: {0:s}, value: BootExecute: unsupported value data type: '
            '{1:s}.').format(
                registry_key.path, registry_value.data_type_string)
        parser_mediator.ProduceExtractionWarning(error_string)

      if boot_execute:
        event_data = WindowsBootExecuteEventData()
        event_data.key_path = registry_key.path
        event_data.last_written_time = registry_key.last_written_time
        event_data.value = boot_execute

        parser_mediator.ProduceEventData(event_data)

    self._ProduceDefaultWindowsRegistryEvent(
        parser_mediator, registry_key, names_to_skip=['BootExecute'])


winreg_parser.WinRegistryParser.RegisterPlugins([
    BootVerificationPlugin, BootExecutePlugin])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for PL-SQL recall file parser."""

import unittest

from plaso.parsers import pls_recall

from tests.parsers import test_lib


class PlsRecallTest(test_lib.ParserTestCase):
  """Tests for PL-SQL recall file parser."""

  def testParse(self):
    """Tests the Parse function."""
    parser = pls_recall.PlsRecallParser()
    storage_writer = self._ParseFile(['PLSRecall_Test.dat'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Note that the test file actually has 'test_databae' in the table name.
    expected_event_values = {
        'data_type': 'pls_recall:entry',
        'database_name': 'DB11',
        'query': 'SELECT * from test_databae where date > \'01/01/2012\'',
        'sequence_number': 206,
        'username': 'tsltmp',
        'written_time': '2013-06-18T19:50:00.550000+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for filestat parser."""

import os

import unittest

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory

from plaso.parsers import filestat

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class FileStatTest(test_lib.ParserTestCase):
  """Tests for filestat parser."""

  def testExt4File(self):
    """Test a file in an ext4 file system."""
    test_file_path = self._GetTestFilePath(['ext4.raw'])
    self._SkipIfPathNotExists(test_file_path)

    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    ext_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_EXT, inode=14,
        location='/passwords.txt', parent=os_path_spec)

    parser = filestat.FileStatParser()
    storage_writer = self._ParseFileByPathSpec(ext_path_spec, parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'access_time': '2021-07-22T14:07:32.842610820+00:00',
        'added_time': None,
        'attribute_names': ['security.selinux'],
        'backup_time': None,
        'change_time': '2021-07-22T14:07:32.842610820+00:00',
        'creation_time': '2021-07-22T14:07:32.842610820+00:00',
        'data_type': 'fs:stat',
        'deletion_time': None,
        'display_name': 'EXT:/passwords.txt',
        'file_entry_type': 'file',
        'filename': '/passwords.txt',
        'file_size': 116,
        'file_system_type': 'EXT',
        'group_identifier': 1000,
        'inode': 14,
        'is_allocated': True,
        'mode': 0o664,
        'modification_time': '2021-07-22T14:07:32.842610820+00:00',
        'number_of_links': 1,
        'owner_identifier': 1000}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testExt2FileWithTSK(self):
    """Test a file in an ext2 file system with TSK."""
    test_file_path = self._GetTestFilePath(['ímynd.dd'])
    self._SkipIfPathNotExists(test_file_path)

    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    tsk_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, inode=15,
        location='/passwords.txt', parent=os_path_spec)

    parser = filestat.FileStatParser()
    storage_writer = self._ParseFileByPathSpec(tsk_path_spec, parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Note that attribute support in pytsk/libtsk is currently broken
    # also see: https://github.com/py4n6/pytsk/issues/79
    expected_event_values = {
        'access_time': '2012-05-25T16:00:53+00:00',
        'added_time': None,
        'attribute_names': None,
        'backup_time': None,
        'change_time': '2012-05-25T16:01:03+00:00',
        'creation_time': None,
        'data_type': 'fs:stat',
        'display_name': 'TSK:/passwords.txt',
        'file_entry_type': 'file',
        'file_size': 116,
        'file_system_type': 'EXT2',
        'group_identifier': 5000,
        'inode': 15,
        'mode': 0o400,
        'modification_time': '2012-05-25T16:00:53+00:00',
        'number_of_links': 1,
        'owner_identifier': 151107}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testNtfsFile(self):
    """Test a file in a NTFS file system."""
    test_file_path = self._GetTestFilePath(['vsstest.qcow2'])
    self._SkipIfPathNotExists(test_file_path)

    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    qcow_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_QCOW, parent=os_path_spec)
    ext_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_NTFS,
        location='\\$Extend\\$RmMetadata\\$TxfLog\\$TxfLog.blf',
        parent=qcow_path_spec)

    parser = filestat.FileStatParser()
    storage_writer = self._ParseFileByPathSpec(ext_path_spec, parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'access_time': '2013-12-03T06:30:42.9779097+00:00',
        'added_time': None,
        'attribute_names': None,
        'backup_time': None,
        'change_time': '2013-12-03T06:39:02.0207867+00:00',
        'creation_time': '2013-12-03T06:30:42.9779097+00:00',
        'data_type': 'fs:stat',
        'display_name': 'NTFS:\\$Extend\\$RmMetadata\\$TxfLog\\$TxfLog.blf',
        'file_entry_type': 'file',
        'file_size': 65536,
        'file_system_type': 'NTFS',
        'modification_time': '2013-12-03T06:39:02.0207867+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testZipFile(self):
    """Test a ZIP file."""
    test_file_path = self._GetTestFilePath(['syslog.zip'])
    self._SkipIfPathNotExists(test_file_path)

    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    zip_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_ZIP, location='/syslog',
        parent=os_path_spec)

    parser = filestat.FileStatParser()
    storage_writer = self._ParseFileByPathSpec(zip_path_spec, parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'access_time': None,
        'added_time': None,
        'attribute_names': None,
        'backup_time': None,
        'change_time': None,
        'creation_time': None,
        'data_type': 'fs:stat',
        'display_name': 'ZIP:/syslog',
        'file_entry_type': 'file',
        'file_size': 1247,
        'file_system_type': 'ZIP',
        'inode': None,
        'modification_time': '2012-07-24T14:45:24+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testGzipFile(self):
    """Test a GZIP file."""
    test_file_path = self._GetTestFilePath(['syslog.gz'])
    self._SkipIfPathNotExists(test_file_path)

    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    gzip_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_GZIP, parent=os_path_spec)

    parser = filestat.FileStatParser()
    storage_writer = self._ParseFileByPathSpec(gzip_path_spec, parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    test_path = os.path.join(shared_test_lib.TEST_DATA_PATH, 'syslog.gz')

    expected_event_values = {
        'access_time': None,
        'added_time': None,
        'attribute_names': None,
        'backup_time': None,
        'change_time': None,
        'creation_time': None,
        'data_type': 'fs:stat',
        'display_name': 'GZIP:{0:s}'.format(test_path),
        'file_entry_type': 'file',
        'file_size': 1247,
        'file_system_type': 'GZIP',
        'inode': None,
        'modification_time': '2012-07-28T16:44:07+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testTarFile(self):
    """Test a TAR file."""
    test_file_path = self._GetTestFilePath(['syslog.tar'])
    self._SkipIfPathNotExists(test_file_path)

    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    tar_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TAR, location='/syslog',
        parent=os_path_spec)

    parser = filestat.FileStatParser()
    storage_writer = self._ParseFileByPathSpec(tar_path_spec, parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'access_time': None,
        'added_time': None,
        'attribute_names': None,
        'backup_time': None,
        'change_time': None,
        'creation_time': None,
        'data_type': 'fs:stat',
        'display_name': 'TAR:/syslog',
        'file_entry_type': 'file',
        'file_size': 1247,
        'file_system_type': 'TAR',
        'inode': None,
        'modification_time': '2012-07-24T21:45:24+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testNestedFile(self):
    """Test a nested file."""
    test_file_path = self._GetTestFilePath(['syslog.tgz'])
    self._SkipIfPathNotExists(test_file_path)

    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    gzip_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_GZIP, parent=os_path_spec)
    tar_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TAR, location='/syslog',
        parent=gzip_path_spec)

    parser = filestat.FileStatParser()
    storage_writer = self._ParseFileByPathSpec(tar_path_spec, parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'access_time': None,
        'added_time': None,
        'attribute_names': None,
        'backup_time': None,
        'change_time': None,
        'creation_time': None,
        'data_type': 'fs:stat',
        'display_name': 'TAR:/syslog',
        'file_entry_type': 'file',
        'file_size': 1247,
        'file_system_type': 'TAR',
        'inode': None,
        'modification_time': '2012-07-24T21:45:24+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    test_file_path = self._GetTestFilePath(['syslog.tgz'])
    self._SkipIfPathNotExists(test_file_path)

    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    gzip_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_GZIP, parent=os_path_spec)

    storage_writer = self._ParseFileByPathSpec(gzip_path_spec, parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    test_path = os.path.join(shared_test_lib.TEST_DATA_PATH, 'syslog.tgz')

    expected_event_values = {
        'access_time': None,
        'added_time': None,
        'attribute_names': None,
        'backup_time': None,
        'change_time': None,
        'creation_time': None,
        'data_type': 'fs:stat',
        'display_name': 'GZIP:{0:s}'.format(test_path),
        'file_entry_type': 'file',
        'file_size': 10240,
        'file_system_type': 'GZIP',
        'inode': None,
        'modification_time': '2012-07-28T16:44:43+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testNestedTSK(self):
    """Test a nested TSK file."""
    test_file_path = self._GetTestFilePath(['syslog_image.dd'])
    self._SkipIfPathNotExists(test_file_path)

    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_file_path)
    tsk_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, inode=11,
        location='/logs/hidden.zip', parent=os_path_spec)
    zip_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_ZIP, location='/syslog',
        parent=tsk_path_spec)

    parser = filestat.FileStatParser()
    storage_writer = self._ParseFileByPathSpec(zip_path_spec, parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'access_time': None,
        'added_time': None,
        'attribute_names': None,
        'backup_time': None,
        'change_time': None,
        'creation_time': None,
        'data_type': 'fs:stat',
        'display_name': 'ZIP:/syslog',
        'file_entry_type': 'file',
        'file_size': 1247,
        'file_system_type': 'ZIP',
        'inode': None,
        'modification_time': '2012-07-20T15:44:14+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains the attribute container store serializers."""

import json

from acstore import interface as acstore_interface

from dfdatetime import serializer as dfdatetime_serializer

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as dfvfs_path_spec_factory


class JSONDateTimeAttributeSerializer(acstore_interface.AttributeSerializer):
  """JSON date time values attribute serializer."""

  def DeserializeValue(self, value):
    """Deserializes a value.

    Args:
      value (str): serialized value.

    Returns:
      dfdatetime.DateTimeValues: runtime value.
    """
    json_dict = json.loads(value)

    return dfdatetime_serializer.Serializer.ConvertDictToDateTimeValues(
        json_dict)

  def SerializeValue(self, value):
    """Serializes a value.

    Args:
      value (dfdatetime.DateTimeValues): runtime value.

    Returns:
      list[str]: serialized value.
    """
    json_dict = dfdatetime_serializer.Serializer.ConvertDateTimeValuesToDict(
        value)

    return json.dumps(json_dict)


class JSONPathSpecAttributeSerializer(acstore_interface.AttributeSerializer):
  """JSON path specification attribute serializer."""

  def DeserializeValue(self, value):
    """Deserializes a value.

    Args:
      value (str): serialized value.

    Returns:
      dfvfs.PathSpec: runtime value.
    """
    json_dict = json.loads(value)

    type_indicator = json_dict.get('type_indicator', None)
    if type_indicator:
      del json_dict['type_indicator']

    if 'parent' in json_dict:
      json_dict['parent'] = self.DeserializeValue(json_dict['parent'])

    # Remove the class type from the JSON dictionary since we cannot pass it.
    del json_dict['__type__']

    path_spec = dfvfs_path_spec_factory.Factory.NewPathSpec(
        type_indicator, **json_dict)

    if type_indicator == dfvfs_definitions.TYPE_INDICATOR_OS:
      # dfvfs.OSPathSpec() will change the location to an absolute path
      # here we want to preserve the original location.
      path_spec.location = json_dict.get('location', None)

    return path_spec

  def SerializeValue(self, value):
    """Serializes a value.

    Args:
      value (dfvfs.PathSpec): runtime value.

    Returns:
      str: serialized value.
    """
    json_dict = {'__type__': 'PathSpec'}
    for property_name in dfvfs_path_spec_factory.Factory.PROPERTY_NAMES:
      property_value = getattr(value, property_name, None)
      if property_value is not None:
        json_dict[property_name] = property_value

    if value.HasParent():
      json_dict['parent'] = self.SerializeValue(value.parent)

    json_dict['type_indicator'] = value.type_indicator
    location = getattr(value, 'location', None)
    if location:
      json_dict['location'] = location

    return json.dumps(json_dict)


class JSONStringsListAttributeSerializer(acstore_interface.AttributeSerializer):
  """JSON strings list attribute serializer."""

  def DeserializeValue(self, value):
    """Deserializes a value.

    Args:
      value (str): serialized value.

    Returns:
      list[str]: runtime value.
    """
    return json.loads(value)

  def SerializeValue(self, value):
    """Serializes a value.

    Args:
      value (str): runtime value.

    Returns:
      list[str]: serialized value.
    """
    return json.dumps(value)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Advanced Packaging Tool (APT) History log text plugin."""

import unittest

from dfvfs.helpers import fake_file_system_builder

from plaso.parsers import text_parser
from plaso.parsers.text_plugins import apt_history

from tests.parsers.text_plugins import test_lib


class APTHistoryLogTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the APT History log text parser plugin."""

  def testCheckRequiredFormat(self):
    """Tests for the CheckRequiredFormat method."""
    plugin = apt_history.APTHistoryLogTextPlugin()

    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'Start-Date: 2019-07-10  16:38:08\n'
        b'Commandline: apt-get upgrade --no-install-recommends --assume-yes\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertTrue(result)

  def testProcess(self):
    """Tests the Process function."""
    plugin = apt_history.APTHistoryLogTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['apt_history.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 10)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'command': 'Install',
        'command_line': (
            'apt-get -y install python-pip python3-pip python-dev python3-dev '
            'git tmux screen joe'),
        'data_type': 'linux:apt_history_log:entry',
        'end_time': '2019-07-11T12:21:28',
        'packages': (
            'libmpc3:amd64 (1.0.3-1+b2, automatic), '
            'manpages:amd64 (4.10-2, automatic), '
            'libmpx2:amd64 (6.3.0-18+deb9u1, automatic), '
            'python3-dev:amd64 (3.5.3-1), '
            'python2.7-dev:amd64 (2.7.13-2+deb9u3, automatic), '
            'libdbus-1-3:amd64 (1.10.28-0+deb9u1, automatic), '
            'linux-libc-dev:amd64 (4.9.168-1+deb9u3, automatic), '
            'python-xdg:amd64 (0.25-4, automatic), '
            'libmpfr4:amd64 (3.1.5-1, automatic), '
            'sgml-base:amd64 (1.29, automatic), '
            'libgirepository-1.0-1:amd64 (1.50.0-1+b1, automatic), '
            'libfakeroot:amd64 (1.21-3.1, automatic), '
            'libc6-dev:amd64 (2.24-11+deb9u4, automatic), '
            'rename:amd64 (0.20-4, automatic), '
            'git-man:amd64 (1:2.11.0-3+deb9u4, automatic), '
            'xdg-user-dirs:amd64 (0.15-2+b1, automatic), '
            'libexpat1-dev:amd64 (2.2.0-2+deb9u2, automatic), '
            'cpp-6:amd64 (6.3.0-18+deb9u1, automatic), '
            'dbus:amd64 (1.10.28-0+deb9u1, automatic), '
            'libalgorithm-diff-perl:amd64 (1.19.03-1, automatic), '
            'python-gi:amd64 (3.22.0-2, automatic), '
            'libalgorithm-merge-perl:amd64 (0.08-3, automatic), '
            'libicu57:amd64 (57.1-6+deb9u2, automatic), '
            'binutils:amd64 (2.28-5, automatic), '
            'cpp:amd64 (4:6.3.0-4, automatic), '
            'python-dbus:amd64 (1.2.4-1+b1, automatic), '
            'libitm1:amd64 (6.3.0-18+deb9u1, automatic), '
            'libpython-all-dev:amd64 (2.7.13-2, automatic), '
            'g++:amd64 (4:6.3.0-4, automatic), '
            'perl-modules-5.24:amd64 (5.24.1-3+deb9u5, automatic), '
            'libpython2.7:amd64 (2.7.13-2+deb9u3, automatic), '
            'python3-pip:amd64 (9.0.1-2+deb9u1), '
            'python3-keyring:amd64 (10.1-1, automatic), '
            'python3-wheel:amd64 (0.29.0-2, automatic), '
            'libpython3.5:amd64 (3.5.3-1+deb9u1, automatic), '
            'gcc:amd64 (4:6.3.0-4, automatic), '
            'git:amd64 (1:2.11.0-3+deb9u4), '
            'python3-idna:amd64 (2.2-1, automatic), '
            'libpython2.7-dev:amd64 (2.7.13-2+deb9u3, automatic), '
            'libcilkrts5:amd64 (6.3.0-18+deb9u1, automatic), '
            'python3-six:amd64 (1.10.0-3, automatic), '
            'libasan3:amd64 (6.3.0-18+deb9u1, automatic), '
            'libquadmath0:amd64 (6.3.0-18+deb9u1, automatic), '
            'python3.5-dev:amd64 (3.5.3-1+deb9u1, automatic), '
            'joe:amd64 (4.4-1), '
            'libisl15:amd64 (0.18-1, automatic), '
            'python-enum34:amd64 (1.1.6-1, automatic), '
            'build-essential:amd64 (12.3, automatic), '
            'libfile-fcntllock-perl:amd64 (0.22-3+b2, automatic), '
            'python3-xdg:amd64 (0.25-4, automatic), '
            'libperl5.24:amd64 (5.24.1-3+deb9u5, automatic), '
            'python-cryptography:amd64 (1.7.1-3+deb9u1, automatic), '
            'python-crypto:amd64 (2.6.1-7, automatic), '
            'xml-core:amd64 (0.17, automatic), '
            'python-keyrings.alt:amd64 (1.3-1, automatic), '
            'python-cffi-backend:amd64 (1.9.1-2, automatic), '
            'python-ipaddress:amd64 (1.0.17-1, automatic), '
            'libtsan0:amd64 (6.3.0-18+deb9u1, automatic), '
            'libgcc-6-dev:amd64 (6.3.0-18+deb9u1, automatic), '
            'python3-cryptography:amd64 (1.7.1-3+deb9u1, automatic), '
            'libubsan0:amd64 (6.3.0-18+deb9u1, automatic), '
            'gir1.2-glib-2.0:amd64 (1.50.0-1+b1, automatic), '
            'g++-6:amd64 (6.3.0-18+deb9u1, automatic), '
            'libpython-dev:amd64 (2.7.13-2, automatic), '
            'python3-cffi-backend:amd64 (1.9.1-2, automatic), '
            'make:amd64 (4.1-9.1, automatic), '
            'fakeroot:amd64 (1.21-3.1, automatic), '
            'gcc-6:amd64 (6.3.0-18+deb9u1, automatic), '
            'python-all:amd64 (2.7.13-2, automatic), '
            'liblsan0:amd64 (6.3.0-18+deb9u1, automatic), '
            'shared-mime-info:amd64 (1.8-1+deb9u1, automatic), '
            'libgomp1:amd64 (6.3.0-18+deb9u1, automatic), '
            'libpython3.5-dev:amd64 (3.5.3-1+deb9u1, automatic), '
            'libdbus-glib-1-2:amd64 (0.108-2, automatic), '
            'libutempter0:amd64 (1.1.6-3, automatic), '
            'python-secretstorage:amd64 (2.3.1-2, automatic), '
            'python-dev:amd64 (2.7.13-2), '
            'python-pyasn1:amd64 (0.1.9-2, automatic), '
            'python-setuptools:amd64 (33.1.1-1, automatic), '
            'python-keyring:amd64 (10.1-1, automatic), '
            'manpages-dev:amd64 (4.10-2, automatic), '
            'bzip2:amd64 (1.0.6-8.1, automatic), '
            'python3-dbus:amd64 (1.2.4-1+b1, automatic), '
            'libevent-2.0-5:amd64 (2.0.21-stable-3, automatic), '
            'libglib2.0-data:amd64 (2.50.3-2, automatic), '
            'python3-keyrings.alt:amd64 (1.3-1, automatic), '
            'libc-dev-bin:amd64 (2.24-11+deb9u4, automatic), '
            'libxml2:amd64 (2.9.4+dfsg1-2.2+deb9u2, automatic), '
            'python3-gi:amd64 (3.22.0-2, automatic), '
            'python3-crypto:amd64 (2.6.1-7, automatic), '
            'perl:amd64 (5.24.1-3+deb9u5, automatic), '
            'rsync:amd64 (3.1.2-1+deb9u2, automatic), '
            'tmux:amd64 (2.3-4), '
            'python-idna:amd64 (2.2-1, automatic), '
            'python-wheel:amd64 (0.29.0-2, automatic), '
            'libpython3-dev:amd64 (3.5.3-1, automatic), '
            'libatomic1:amd64 (6.3.0-18+deb9u1, automatic), '
            'libcc1-0:amd64 (6.3.0-18+deb9u1, automatic), '
            'libdpkg-perl:amd64 (1.18.25, automatic), '
            'patch:amd64 (2.7.5-1+deb9u1, automatic), '
            'python3-secretstorage:amd64 (2.3.1-2, automatic), '
            'libalgorithm-diff-xs-perl:amd64 (0.04-4+b2, automatic), '
            'python-pip-whl:amd64 (9.0.1-2+deb9u1, automatic), '
            'libglib2.0-0:amd64 (2.50.3-2, automatic), '
            'python-pip:amd64 (9.0.1-2+deb9u1), '
            'libcurl3-gnutls:amd64 (7.52.1-5+deb9u9, automatic), '
            'less:amd64 (481-2.1, automatic), '
            'python3-setuptools:amd64 (33.1.1-1, automatic), '
            'dpkg-dev:amd64 (1.18.25, automatic), '
            'python-all-dev:amd64 (2.7.13-2, automatic), '
            'python3-pyasn1:amd64 (0.1.9-2, automatic), '
            'libstdc++-6-dev:amd64 (6.3.0-18+deb9u1, automatic), '
            'liberror-perl:amd64 (0.17024-1, automatic)'),
        'start_time': '2019-07-11T12:20:55'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the event object filter."""

import unittest

from plaso.containers import events
from plaso.filters import event_filter
from plaso.lib import errors

from tests.filters import test_lib


class EventObjectFilterTest(test_lib.FilterTestCase):
  """Tests for the event object filter."""

  def testCompilerFilter(self):
    """Tests the CompileFilter function."""
    test_filter = event_filter.EventObjectFilter()

    test_filter.CompileFilter(
        'some_stuff is "random" and other_stuff is not "random"')

    test_filter.CompileFilter('timestamp is "2020-12-23 15:00:00"')

    test_filter.CompileFilter('timestamp is DATETIME("2020-12-23T15:00:00")')

    test_filter.CompileFilter('filename contains PATH("/etc/issue")')

    with self.assertRaises(errors.ParseError):
      test_filter.CompileFilter(
          'SELECT stuff FROM machine WHERE conditions are met')

    with self.assertRaises(errors.ParseError):
      test_filter.CompileFilter(
          '/tmp/file_that_most_likely_does_not_exist')

    with self.assertRaises(errors.ParseError):
      test_filter.CompileFilter(
          'some random stuff that is destined to fail')

    with self.assertRaises(errors.ParseError):
      test_filter.CompileFilter(
          'some_stuff is "random" and other_stuff ')

  def testMatch(self):
    """Tests the Match function."""
    test_filter = event_filter.EventObjectFilter()
    test_filter.CompileFilter('timestamp is DATETIME("2020-12-23T15:00:00")')

    event = events.EventObject()
    event.timestamp = 1608735600000000

    result = test_filter.Match(event, None, None, None)
    self.assertTrue(result)

    test_filter = event_filter.EventObjectFilter()
    test_filter.CompileFilter('filename contains PATH("etc/issue")')

    event_data = events.EventData()
    event_data.filename = '/usr/local/etc/issue'

    result = test_filter.Match(None, event_data, None, None)
    self.assertTrue(result)

    event_data.filename = '/etc/issue.net'

    result = test_filter.Match(None, event_data, None, None)
    self.assertFalse(result)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for Fish History file parser."""

import unittest

from plaso.parsers import fish_history

from tests.parsers import test_lib


class FishHistoryTest(test_lib.ParserTestCase):
  """Tests for Fish History file parser."""

  def testParseFile(self):
    """Test parsing of a Fish History file."""
    parser = fish_history.FishHistoryParser()
    storage_writer = self._ParseFile(['fish_history'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 10)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'command': 'll',
        'data_type': 'fish:history:entry',
        'written_time': '2021-04-29T22:53:00+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Android SMS call history plugin."""

import unittest

from plaso.parsers.sqlite_plugins import android_calls

from tests.parsers.sqlite_plugins import test_lib


class AndroidCallSQLitePluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Android Call History database plugin."""

  def testProcess(self):
    """Test the Process function on an Android contacts2.db file."""
    plugin = android_calls.AndroidCallPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(['contacts2.db'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 3)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'call_type': 3,
        'data_type': 'android:event:call',
        'duration': 0,
        'end_time': None,
        'number': '5404561685',
        'start_time': '2013-11-06T21:17:16.690+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Text log parser."""

import codecs
import io
import os

import pysigscan

from plaso.lib import errors
from plaso.parsers import interface
from plaso.parsers import logger
from plaso.parsers import manager


class EncodedTextReader(object):
  """Encoded text reader.

  Attributes:
    line_number (int): current line number.
    lines (str): lines of text.
    lines_size (int): size of the lines of text.
  """

  BUFFER_SIZE = 65536

  _READ_BUFFER_SIZE = 16 * BUFFER_SIZE

  def __init__(
      self, file_object, encoding='utf-8', encoding_errors='strict'):
    """Initializes the encoded text reader object.

    Args:
      file_object (FileIO): a file-like object to read from.
      encoding (Optional[str]): text encoding.
      encoding_errors (Optional[str]): text encoding errors handler.
    """
    stream_reader_class = codecs.getreader(encoding)

    super(EncodedTextReader, self).__init__()
    self._file_object = file_object
    self._stream_reader = stream_reader_class(
        file_object, errors=encoding_errors)

    self.line_number = 0
    self.lines = ''
    self.lines_size = 0

  def ReadLine(self):
    """Reads a line.

    Returns:
      str: line read from the lines buffer.
    """
    if not self.lines:
      self.ReadLines()

    line, _, self.lines = self.lines.partition('\n')
    self.lines_size += len(line) + 1
    self.line_number += 1

    return line

  def ReadLines(self):
    """Reads lines into the lines buffer."""
    if self.lines_size < self.BUFFER_SIZE:
      current_offset = self._file_object.tell()

      # Consequative reads, decodes and joins are expensive hence we read
      # a larger buffer at once.
      decoded_data = self._stream_reader.read(size=self._READ_BUFFER_SIZE)
      if decoded_data:
        # Remove a byte-order mark at the start of the file.
        if current_offset == 0 and decoded_data[0] == '\ufeff':
          decoded_data = decoded_data[1:]

        # Strip carriage returns from the text.
        decoded_data = '\n'.join([
            line.rstrip('\r') for line in decoded_data.split('\n')])

        self.lines = ''.join([self.lines, decoded_data])
        self.lines_size += len(decoded_data)

  def SkipAhead(self, number_of_characters):
    """Skips ahead a number of characters.

    Args:
      number_of_characters (int): number of characters.
    """
    while number_of_characters >= self.lines_size:
      number_of_characters -= self.lines_size

      self.lines = ''
      self.lines_size = 0

      self.ReadLines()

      if self.lines_size == 0:
        return

    self.line_number += self.lines[:number_of_characters].count('\n')
    self.lines = self.lines[number_of_characters:]
    self.lines_size -= number_of_characters

  # Note: that the following functions do not follow the style guide
  # because they are part of the file-like object interface.
  # pylint: disable=invalid-name

  def get_offset(self):
    """Retrieves the current offset into the file-like object.

    Returns:
      int: current offset into the file-like object.
    """
    return self._file_object.tell()


class TextLogParser(interface.FileObjectParser):
  """Text-based log file parser."""

  NAME = 'text'
  DATA_FORMAT = 'text-based log file'

  _NON_TEXT_CHARACTERS = frozenset([
      '\x00', '\x01', '\x02', '\x03', '\x04', '\x05', '\x06', '\x0b', '\x0e',
      '\x0f', '\x10', '\x11', '\x12', '\x13', '\x14', '\x15', '\x16', '\x17',
      '\x18', '\x19', '\x1a', '\x1c', '\x1d', '\x1e', '\x1f', '\x7f'])

  _plugin_classes = {}

  def __init__(self):
    """Initializes a text-based log parser."""
    super(TextLogParser, self).__init__()
    self._plugins_per_encoding = {}
    self._format_scanner = None
    self._non_sigscan_plugin_names = None
    self._plugin_name_per_format_identifier = {}

  def _ContainsBinary(self, text):
    """Determines if the text contains binary (non-text) characters.

    Args:
      text (str): text.

    Returns:
      bool: True if the text contains binary (non-text) characters.
    """
    return bool(self._NON_TEXT_CHARACTERS.intersection(set(text)))

  def _CreateFormatScanner(self, parser_mediator):
    """Creates a signature scanner for required format check.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
    """
    self._non_sigscan_plugin_names = set()
    self._plugin_name_per_format_identifier = {}

    scanner_object = pysigscan.scanner()
    scanner_object.set_scan_buffer_size(65536)

    for plugin_name, plugin in self._plugins_per_name.items():
      if not plugin.VERIFICATION_LITERALS:
        self._non_sigscan_plugin_names.add(plugin_name)
      else:
        encoding = plugin.ENCODING
        if not encoding:
          encoding = parser_mediator.GetCodePage()

        for index, literal in enumerate(plugin.VERIFICATION_LITERALS):
          identifier = '{0:s}{1:d}'.format(plugin_name, index)
          encoded_literal = literal.encode(encoding)
          scanner_object.add_signature(
              identifier, 0, encoded_literal,
              pysigscan.signature_flags.NO_OFFSET)

          self._plugin_name_per_format_identifier[identifier] = plugin_name

    if self._plugin_name_per_format_identifier:
      self._format_scanner = scanner_object

  def EnablePlugins(self, plugin_includes):
    """Enables parser plugins.

    Args:
      plugin_includes (set[str]): names of the plugins to enable, where
          set(['*']) represents all plugins. Note the default plugin, if
          it exists, is always enabled and cannot be disabled.
    """
    self._plugins_per_name = {}
    self._plugins_per_encoding = {}
    if not self._plugin_classes:
      return

    for plugin_name, plugin_class in self._plugin_classes.items():
      if plugin_name == self._default_plugin_name:
        self._default_plugin = plugin_class()
        continue

      if (plugin_includes != self.ALL_PLUGINS and
          plugin_name not in plugin_includes):
        continue

      plugin_object = plugin_class()
      self._plugins_per_name[plugin_name] = plugin_object

      encoding = plugin_class.ENCODING or 'default'
      if encoding not in self._plugins_per_encoding:
        self._plugins_per_encoding[encoding] = []

      self._plugins_per_encoding[encoding].append(plugin_object)

  def ParseFileObject(self, parser_mediator, file_object):
    """Parses a text log file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      file_object (dfvfs.FileIO): file-like object.

    Raises:
      WrongParser: when the file cannot be parsed.
    """
    if not self._format_scanner and not self._non_sigscan_plugin_names:
      self._CreateFormatScanner(parser_mediator)

    file_object.seek(0, os.SEEK_SET)

    # Cache the first 64k of encoded data so it does not need to be read for
    # each encoding.
    encoded_data_buffer = file_object.read(EncodedTextReader.BUFFER_SIZE)
    encoded_data_file_object = io.BytesIO(encoded_data_buffer)

    plugins_with_matching_literals = set()
    if self._format_scanner:
      parser_mediator.SampleFormatCheckStartTiming('text_format_scanner')

      try:
        scan_state = pysigscan.scan_state()
        self._format_scanner.scan_file_object(
            scan_state, encoded_data_file_object)

        for scan_result in iter(scan_state.scan_results):
          plugin_name = self._plugin_name_per_format_identifier.get(
              scan_result.identifier, None)

          plugins_with_matching_literals.add(plugin_name)

      finally:
        parser_mediator.SampleFormatCheckStopTiming('text_format_scanner')

    matching_plugin = False
    for encoding, plugins in self._plugins_per_encoding.items():
      if parser_mediator.abort:
        break

      if encoding == 'default':
        encoding = parser_mediator.GetCodePage()

      text_reader = None

      for plugin in plugins:
        if parser_mediator.abort:
          break

        profiling_name = '/'.join([self.NAME, plugin.NAME])

        parser_mediator.SampleFormatCheckStartTiming(profiling_name)

        try:
          logger.debug(
              'Checking required format of: {0:s} in encoding: {1:s}'.format(
                  plugin.NAME, encoding))

          result = False
          if (plugin.NAME in plugins_with_matching_literals or
              plugin.NAME in self._non_sigscan_plugin_names):
            if not text_reader:
              encoded_data_file_object.seek(0, os.SEEK_SET)
              text_reader = EncodedTextReader(
                  encoded_data_file_object, encoding=encoding)

              text_reader.ReadLines()

              # TODO: check if this works with xchatscrollback log.
              if self._ContainsBinary(text_reader.lines):
                logger.debug('Detected binary format')
                continue

            result = plugin.CheckRequiredFormat(parser_mediator, text_reader)

        except UnicodeDecodeError:
          logger.debug(
              'Unable to read text-based log file with encoding: {0:s}'.format(
                  encoding))
          result = False

        finally:
          parser_mediator.SampleFormatCheckStopTiming(profiling_name)

        if result:
          matching_plugin = True

          parser_mediator.SampleStartTiming(profiling_name)

          try:
            plugin.UpdateChainAndProcess(
                parser_mediator, file_object=file_object)
          except Exception as exception:  # pylint: disable=broad-except
            parser_mediator.ProduceExtractionWarning((
                'plugin: {0:s} unable to parse text file with error: '
                '{1!s}').format(plugin.NAME, exception))
            continue

          finally:
            parser_mediator.SampleStopTiming(profiling_name)

          if hasattr(plugin, 'GetYearLessLogHelper'):
            year_less_log_helper = plugin.GetYearLessLogHelper()
            parser_mediator.AddYearLessLogHelper(year_less_log_helper)

          break

      if matching_plugin:
        break

    if not matching_plugin:
      raise errors.WrongParser('No matching text-based log plugin found.')


manager.ParsersManager.RegisterParser(TextLogParser)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Windows Timeline SQLite database plugin."""

import unittest

from plaso.parsers.sqlite_plugins import windows_timeline

from tests.parsers.sqlite_plugins import test_lib


class WindowsTimelineTest(test_lib.SQLitePluginTestCase):
  """Tests for the Windows Timeline plugin."""

  def testProcess(self):
    """Tests the Process function on a Windows Timeline SQLite database."""
    plugin = windows_timeline.WindowsTimelinePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['windows_timeline_ActivitiesCache.db'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 112)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'active_duration_seconds': 9,
        'data_type': 'windows:timeline:user_engaged',
        'package_identifier': 'c:\\python34\\python.exe',
        'reporting_app': 'ShellActivityMonitor',
        'start_time': '2018-08-03T11:29:00+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    expected_event_values = {
        'application_display_name': 'OneDrive',
        'data_type': 'windows:timeline:generic',
        'description': None,
        'package_identifier': 'Microsoft.SkyDrive.Desktop',
        'start_time': '2018-07-25T12:04:48+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 80)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The output mediator object."""

import glob
import os
import pytz

from plaso.engine import path_helper
from plaso.formatters import manager as formatters_manager
from plaso.formatters import yaml_formatters_file
from plaso.helpers.windows import languages
from plaso.lib import definitions
from plaso.output import winevt_rc


class OutputMediator(object):
  """Output mediator.

  Attributes:
    data_location (Optional[str]): path of the formatter data files.
  """

  _DEFAULT_ENCODING = 'utf-8'

  # The default LCID 0x0409, which represents en-US.
  _DEFAULT_LCID = 0x0409

  _DEFAULT_TIME_ZONE = pytz.UTC

  _WINEVT_RC_DATABASE = 'winevt-rc.db'

  def __init__(
      self, storage_reader, data_location=None, dynamic_time=False,
      preferred_encoding='utf-8'):
    """Initializes an output mediator.

    Args:
      storage_reader (StorageReader): storage reader.
      data_location (Optional[str]): path of the formatter data files.
      dynamic_time (Optional[bool]): True if date and time values should be
          represented in their granularity or semantically.
      preferred_encoding (Optional[str]): preferred encoding to output.
    """
    super(OutputMediator, self).__init__()
    self._dynamic_time = dynamic_time
    self._hostname = None
    self._language_tag = None
    self._lcid = None
    self._message_formatters = {}
    self._preferred_encoding = preferred_encoding
    self._source_mappings = {}
    self._storage_reader = storage_reader
    self._system_configurations = None
    self._time_zone = None
    self._username_by_identifier = {}

    self.data_location = data_location

  @property
  def dynamic_time(self):
    """bool: True if dynamic time should be used."""
    return self._dynamic_time

  @property
  def encoding(self):
    """str: preferred encoding to output."""
    return self._preferred_encoding or self._DEFAULT_ENCODING

  @property
  def time_zone(self):
    """datetime.tzinfo: time zone."""
    return self._time_zone or self._DEFAULT_TIME_ZONE

  def _ReadMessageFormattersFile(self, path, override_existing=False):
    """Reads a message formatters configuration file.

    Args:
      path (str): path of file that contains the message formatters
          configuration.
      override_existing (bool): True if existing message formatters should
          be overridden.

    Raises:
      KeyError: if the message formatter is already set for the corresponding
          data type and override existing is False.
    """
    message_formatters_file = yaml_formatters_file.YAMLFormattersFile()
    for message_formatter in message_formatters_file.ReadFromFile(path):
      if (message_formatter.data_type in self._message_formatters and
          not override_existing):
        raise KeyError(
            'Message formatter for data type: {0:s} already exists'.format(
                message_formatter.data_type))

      for identifier in message_formatter.custom_helpers:
        custom_formatter_helper = (
             formatters_manager.FormattersManager.GetEventFormatterHelper(
                identifier))
        if custom_formatter_helper:
          message_formatter.AddHelper(custom_formatter_helper)

      self._message_formatters[message_formatter.data_type] = message_formatter
      self._source_mappings[message_formatter.data_type] = (
          message_formatter.source_mapping)

  def _ReadUserAccount(self, user_identifier):
    """Reads a specific user account from the storage.

    Args:
      user_identifier (str): user identifier (UID or SID).

    Returns:
      UserAccountArtifact: user account or None if not available.
    """
    # TODO: get username related to the source.
    filter_expression = 'identifier == "{0:s}"'.format(user_identifier)
    user_accounts = list(self._storage_reader.GetAttributeContainers(
        'user_account', filter_expression=filter_expression))

    if not user_accounts:
      return None

    return user_accounts[0]

  def GetDisplayNameForPathSpec(self, path_spec):
    """Retrieves the display name for a path specification.

    Args:
      path_spec (dfvfs.PathSpec): path specification.

    Returns:
      str: human readable version of the path specification.
    """
    return path_helper.PathHelper.GetDisplayNameForPathSpec(path_spec)

  def GetHostname(self, event_data, default_hostname='-'):
    """Retrieves the hostname related to the event.

    Args:
      event_data (EventData): event data.
      default_hostname (Optional[str]): default hostname.

    Returns:
      str: hostname.
    """
    hostname = getattr(event_data, 'hostname', None)
    if hostname:
      return hostname

    if self._system_configurations is None:
      self._system_configurations = list(
          self._storage_reader.GetAttributeContainers('system_configuration'))

    # TODO: get hostname related to the source.
    if not self._hostname and self._system_configurations:
      hostname_artifact = self._system_configurations[-1].hostname
      if hostname_artifact:
        self._hostname = hostname_artifact.name

    return self._hostname or default_hostname

  def GetMACBRepresentation(self, event, event_data):
    """Retrieves the MACB representation.

    Args:
      event (EventObject): event.
      event_data (EventData): event data.

    Returns:
      str: MACB representation.
    """
    data_type = getattr(event_data, 'data_type', None)
    if not data_type:
      return '....'

    # The filestat parser is somewhat limited.
    if data_type == 'fs:stat':
      descriptions = event.timestamp_desc.split(';')

      return_characters = ['.', '.', '.', '.']
      for description in descriptions:
        if description in (
            'mtime', definitions.TIME_DESCRIPTION_MODIFICATION):
          return_characters[0] = 'M'
        elif description in (
            'atime', definitions.TIME_DESCRIPTION_LAST_ACCESS):
          return_characters[1] = 'A'
        elif description in (
            'ctime', definitions.TIME_DESCRIPTION_METADATA_MODIFICATION):
          return_characters[2] = 'C'
        elif description in (
            'crtime', definitions.TIME_DESCRIPTION_CREATION):
          return_characters[3] = 'B'

      return ''.join(return_characters)

    # Access time.
    if event.timestamp_desc in [
        definitions.TIME_DESCRIPTION_LAST_ACCESS,
        definitions.TIME_DESCRIPTION_ACCOUNT_CREATED,
        definitions.TIME_DESCRIPTION_LAST_VISITED,
        definitions.TIME_DESCRIPTION_START,
        definitions.TIME_DESCRIPTION_LAST_SHUTDOWN,
        definitions.TIME_DESCRIPTION_LAST_LOGIN,
        definitions.TIME_DESCRIPTION_LAST_PASSWORD_RESET,
        definitions.TIME_DESCRIPTION_LAST_CONNECTED,
        definitions.TIME_DESCRIPTION_LAST_RUN,
        definitions.TIME_DESCRIPTION_LAST_PRINTED]:
      return '.A..'

    # Content modification.
    if event.timestamp_desc in [
        definitions.TIME_DESCRIPTION_MODIFICATION,
        definitions.TIME_DESCRIPTION_WRITTEN,
        definitions.TIME_DESCRIPTION_DELETED]:
      return 'M...'

    # Content creation time.
    if event.timestamp_desc in [
        definitions.TIME_DESCRIPTION_CREATION,
        definitions.TIME_DESCRIPTION_ADDED,
        definitions.TIME_DESCRIPTION_FILE_DOWNLOADED,
        definitions.TIME_DESCRIPTION_FIRST_CONNECTED]:
      return '...B'

    # Metadata modification.
    if event.timestamp_desc == (
        definitions.TIME_DESCRIPTION_METADATA_MODIFICATION):
      return '..C.'

    return '....'

  def GetMACBRepresentationFromDescriptions(self, timestamp_descriptions):
    """Determines the MACB representation from the timestamp descriptions.

    MACB representation is a shorthand for representing one or more of
    modification, access, change, birth timestamp descriptions as the letters
    "MACB" or a "." if the corresponding timestamp is not set.

    Note that this is an output format shorthand and does not guarantee that
    the timestamps represent the same occurrence.

    Args:
      timestamp_descriptions (list[str]): timestamp descriptions, which are
          defined in definitions.TIME_DESCRIPTIONS.

    Returns:
      str: MACB representation.
    """
    macb_representation = []

    if ('mtime' in timestamp_descriptions or
        definitions.TIME_DESCRIPTION_MODIFICATION in timestamp_descriptions):
      macb_representation.append('M')
    else:
      macb_representation.append('.')

    if ('atime' in timestamp_descriptions or
        definitions.TIME_DESCRIPTION_LAST_ACCESS in timestamp_descriptions):
      macb_representation.append('A')
    else:
      macb_representation.append('.')

    if ('ctime' in timestamp_descriptions or
        definitions.TIME_DESCRIPTION_METADATA_MODIFICATION in (
            timestamp_descriptions)):
      macb_representation.append('C')
    else:
      macb_representation.append('.')

    if ('crtime' in timestamp_descriptions or
        definitions.TIME_DESCRIPTION_CREATION in timestamp_descriptions):
      macb_representation.append('B')
    else:
      macb_representation.append('.')

    return ''.join(macb_representation)

  def GetMessageFormatter(self, data_type):
    """Retrieves the message formatter for a specific data type.

    Args:
      data_type (str): data type.

    Returns:
      EventFormatter: corresponding message formatter or the default message
          formatter if not available.
    """
    data_type = data_type.lower()
    return self._message_formatters.get(data_type, None)

  def GetRelativePathForPathSpec(self, path_spec):
    """Retrieves the relative path for a path specification.

    Args:
      path_spec (dfvfs.PathSpec): path specification.

    Returns:
      str: relateive path of the path specification.
    """
    return path_helper.PathHelper.GetRelativePathForPathSpec(path_spec)

  def GetSourceMapping(self, data_type):
    """Retrieves the source mapping for a specific data type.

    Args:
      data_type (str): data type.

    Returns:
      tuple[str, str]: short and (long) source mappings or (None, None) if not
          available.
    """
    data_type = data_type.lower()
    return self._source_mappings.get(data_type, (None, None))

  def GetUsername(self, event_data, default_username='-'):
    """Retrieves the username related to the event data.

    Args:
      event_data (EventData): event data.
      default_username (Optional[str]): default username.

    Returns:
      str: username.
    """
    username = getattr(event_data, 'username', None)
    if username and username != '-':
      return username

    username = default_username

    user_identifier = getattr(event_data, 'user_sid', None)
    if (user_identifier and
        user_identifier not in self._username_by_identifier):
      user_account = self._ReadUserAccount(user_identifier)
      if user_account:
        username = user_account.username
        self._username_by_identifier[user_identifier] = username

    return username or default_username

  def GetWinevtResourcesHelper(self):
    """Retrieves a Windows EventLog resources helper.

    Returns:
      WinevtResourcesHelper: Windows EventLog resources helper.
    """
    lcid = self._lcid
    if not lcid:
      # TODO: determine LCID from system configurations
      pass
    if not lcid:
      lcid = self._DEFAULT_LCID

    return winevt_rc.WinevtResourcesHelper(
        self._storage_reader, self.data_location, lcid)

  def ReadMessageFormattersFromDirectory(self, path):
    """Reads message formatters from a directory.

    Args:
      path (str): path of directory that contains the message formatters
          configuration files.

    Raises:
      KeyError: if the message formatter is already set for the corresponding
          data type.
    """
    for formatters_file_path in glob.glob(os.path.join(path, '*.yaml')):
      self._ReadMessageFormattersFile(formatters_file_path)

  def ReadMessageFormattersFromFile(self, path, override_existing=False):
    """Reads message formatters from a file.

    Args:
      path (str): path of file that contains the message formatters
          configuration.
      override_existing (bool): True if existing message formatters should
          be overridden.

    Raises:
      KeyError: if the message formatter is already set for the corresponding
          data type.
    """
    self._ReadMessageFormattersFile(path, override_existing=override_existing)

  def SetPreferredLanguageIdentifier(self, language_tag):
    """Sets the preferred language identifier.

    Args:
      language_tag (str): language tag such as "en-US" for US English or
          "is-IS" for Icelandic.

    Raises:
      ValueError: if the language tag is not a string type or no LCID can
          be determined that corresponds with the language tag.
    """
    lcid = None
    if language_tag:
      if not isinstance(language_tag, str):
        raise ValueError('Language tag: {0!s} is not a string.'.format(
            language_tag))

      lcid = languages.WindowsLanguageHelper.GetLCIDForLanguageTag(language_tag)
      if not lcid:
        raise ValueError('No LCID found for language tag: {0:s}.'.format(
            language_tag))

    self._language_tag = language_tag
    self._lcid = lcid

  def SetTimeZone(self, time_zone):
    """Sets the time zone.

    Args:
      time_zone (str): time zone.

    Raises:
      ValueError: if the time zone is not supported.
    """
    if time_zone:
      try:
        time_zone = pytz.timezone(time_zone)
      except pytz.UnknownTimeZoneError:
        raise ValueError('Unsupported time zone: {0:s}'.format(time_zone))

    self._time_zone = time_zone



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Networks Windows Registry plugin."""

import unittest

from dfdatetime import filetime as dfdatetime_filetime
from dfwinreg import definitions as dfwinreg_definitions
from dfwinreg import fake as dfwinreg_fake

from plaso.parsers.winreg_plugins import networks

from tests.parsers.winreg_plugins import test_lib


class NetworksWindowsRegistryPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the Networks Windows Registry plugin."""

  def _CreateTestKey(self, key_path, time_string):
    """Creates Registry keys and values for testing.

    Args:
      key_path (str): Windows Registry key path.
      time_string (str): key last written date and time.

    Returns:
      dfwinreg.WinRegistryKey: a Windows Registry key.
    """
    filetime = dfdatetime_filetime.Filetime()
    filetime.CopyFromDateTimeString(time_string)
    registry_key = dfwinreg_fake.FakeWinRegistryKey(
        'NetworkList', key_path=key_path,
        last_written_time=filetime.timestamp, offset=153)

    # Setup Profiles.
    profiles_key_name = 'Profiles'
    profiles_key = dfwinreg_fake.FakeWinRegistryKey(profiles_key_name)
    registry_key.AddSubkey(profiles_key_name, profiles_key)

    profile1_key_name = '{B358E985-4464-4ABD-AF99-7D4A0AF66BB7}'
    profile1_key = dfwinreg_fake.FakeWinRegistryKey(profile1_key_name)
    profiles_key.AddSubkey(profile1_key_name, profile1_key)

    value_data = b'\x00\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Category', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    profile1_key.AddValue(registry_value)

    value_data = (
        b'\xde\x07\x0c\x00\x02\x00\x10\x00\x08\x00\x04\x00\x27\x00\x6a\x00')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DateCreated', data=value_data,
        data_type=dfwinreg_definitions.REG_BINARY)
    profile1_key.AddValue(registry_value)

    value_data = (
        b'\xdf\x07\x01\x00\x02\x00\x1b\x00\x0f\x00\x0f\x00\x1b\x00\xc5\x03')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DateLastConnected', data=value_data,
        data_type=dfwinreg_definitions.REG_BINARY)
    profile1_key.AddValue(registry_value)

    value_data = 'My Awesome Wifi Hotspot'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Description', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    profile1_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Managed', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    profile1_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x47'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'NameType', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    profile1_key.AddValue(registry_value)

    value_data = 'My Awesome Wifi Hotspot'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ProfileName', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    profile1_key.AddValue(registry_value)

    profile2_key_name = '{C1C57B58-BFE2-428B-818C-9D69A873AD3D}'
    profile2_key = dfwinreg_fake.FakeWinRegistryKey(profile2_key_name)
    profiles_key.AddSubkey(profile2_key_name, profile2_key)

    value_data = b'\x00\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Category', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    profile2_key.AddValue(registry_value)

    value_data = (
        b'\xde\x07\x05\x00\x02\x00\x06\x00\x11\x00\x02\x00\x13\x00\x1b\x03')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DateCreated', data=value_data,
        data_type=dfwinreg_definitions.REG_BINARY)
    profile2_key.AddValue(registry_value)

    value_data = (
        b'\xde\x07\x05\x00\x02\x00\x06\x00\x11\x00\x07\x00\x36\x00\x0a\x00')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DateLastConnected', data=value_data,
        data_type=dfwinreg_definitions.REG_BINARY)
    profile2_key.AddValue(registry_value)

    value_data = 'Network'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Description', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    profile2_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x00'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Managed', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    profile2_key.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x06'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'NameType', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    profile2_key.AddValue(registry_value)

    value_data = 'Network'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ProfileName', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    profile2_key.AddValue(registry_value)

    # Setup signatures.
    signatures_key_name = 'Signatures'
    signatures_key = dfwinreg_fake.FakeWinRegistryKey(signatures_key_name)
    registry_key.AddSubkey(signatures_key_name, signatures_key)

    managed_key_name = 'Managed'
    managed_key = dfwinreg_fake.FakeWinRegistryKey(managed_key_name)
    signatures_key.AddSubkey(managed_key_name, managed_key)

    unmanaged_key_name = 'Unmanaged'
    unmanaged_key = dfwinreg_fake.FakeWinRegistryKey(unmanaged_key_name)
    signatures_key.AddSubkey(unmanaged_key_name, unmanaged_key)

    unmanaged_subkey_name = (
        '010103000F0000F0080000000F0000F0E8982FB31F37E52AF30A6575A4898CE667'
        '6E8C2F99C4C5131D84F64BD823E0')
    unmanaged_subkey = dfwinreg_fake.FakeWinRegistryKey(unmanaged_subkey_name)
    unmanaged_key.AddSubkey(unmanaged_subkey_name, unmanaged_subkey)

    value_data = b'\x00\x50\x56\xea\x6c\xec'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DefaultGatewayMac', data=value_data,
        data_type=dfwinreg_definitions.REG_BINARY)
    unmanaged_subkey.AddValue(registry_value)

    value_data = 'Network'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Description', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    unmanaged_subkey.AddValue(registry_value)

    value_data = 'localdomain'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'DnsSuffix', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    unmanaged_subkey.AddValue(registry_value)

    value_data = 'Network'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'FirstNetwork', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    unmanaged_subkey.AddValue(registry_value)

    value_data = '{C1C57B58-BFE2-428B-818C-9D69A873AD3D}'.encode('utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'ProfileGuid', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ)
    unmanaged_subkey.AddValue(registry_value)

    value_data = b'\x00\x00\x00\x08'
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        'Source', data=value_data,
        data_type=dfwinreg_definitions.REG_DWORD_BIG_ENDIAN)
    unmanaged_subkey.AddValue(registry_value)

    return registry_key

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = networks.NetworksWindowsRegistryPlugin()

    key_path = (
        'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\CurrentVersion\\'
        'NetworkList')
    self._AssertFiltersOnKeyPath(plugin, key_path)

    self._AssertNotFiltersOnKeyPath(plugin, 'HKEY_LOCAL_MACHINE\\Bogus')

  def testProcess(self):
    """Tests the Process function on created key."""
    key_path = (
        'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\CurrentVersion')
    registry_key = self._CreateTestKey(key_path, '2013-01-30 10:47:57')

    plugin = networks.NetworksWindowsRegistryPlugin()
    storage_writer = self._ParseKeyWithPlugin(registry_key, plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 2)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'connection_type': 6,
        'creation_time': '2014-05-06T17:02:19.795+00:00',
        'data_type': 'windows:registry:network',
        'default_gateway_mac': '00:50:56:ea:6c:ec',
        'description': 'Network',
        'dns_suffix': 'localdomain',
        'last_connected_time': '2014-05-06T17:07:54.010+00:00',
        'ssid': 'Network'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for Program Compatibility Assistant (PCA) log parsers."""

import unittest

from plaso.parsers import winpca
from tests.parsers import test_lib


class WindowsPCADB0ParserTest(test_lib.ParserTestCase):
  """Tests for the Program Compatibility Assistant DB0 log files parser."""

  def testParse(self):
    """Tests the Parse function."""
    plugin = winpca.WindowsPCADB0Parser()
    storage_writer = self._ParseFile(['PcaGeneralDb0.txt'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 3)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'description': 'microsoft teams updater',
        'executable': (
            '%programfiles%\\windowsapps\\microsoftteams_22287'
            '.702.1670.9453_x64__8wekyb3d8bbwe\\msteamsupdate.exe'),
        'exit_code': 'Abnormal process exit with code 0x4c7',
        'last_execution_time': '2022-11-14T23:37:11.789+00:00',
        'program_identifier': '0006132687b1e64961e910ee21d4352afe0800000904',
        'run_status': '2',
        'vendor': 'microsoft corporation',
        'version': '22287.702.1670.9453'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


class WindowsPCADicParserTest(test_lib.ParserTestCase):
  """Tests for the Program Compatibility Assistant DIC log files parser."""

  def testParse(self):
    """Tests the Parse function."""
    plugin = winpca.WindowsPCADicParser()
    storage_writer = self._ParseFile(['PcaAppLaunchDic.txt'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 4)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'executable': (
            'C:\\Program Files\\WindowsApps\\MicrosoftTeams_'
            '22287.702.1670.9453_x64__8wekyb3d8bbwe\\msteams.exe'),
        'last_execution_time': '2022-11-15T00:02:08.476+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Bencode file parser."""

import unittest

from plaso.parsers import bencode_parser
from plaso.parsers import bencode_plugins  # pylint: disable=unused-import

from tests.parsers import test_lib


class BencodeTest(test_lib.ParserTestCase):
  """Tests for the Bencode file parser."""

  # pylint: disable=protected-access

  def testEnablePlugins(self):
    """Tests the EnablePlugins function."""
    parser = bencode_parser.BencodeParser()

    number_of_plugins = len(parser._plugin_classes)

    parser.EnablePlugins([])
    self.assertEqual(len(parser._plugins_per_name), 0)

    parser.EnablePlugins(parser.ALL_PLUGINS)
    self.assertEqual(len(parser._plugins_per_name), number_of_plugins)

    parser.EnablePlugins(['bencode_transmission'])
    self.assertEqual(len(parser._plugins_per_name), 1)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Analysis plugin that labels events according to rules in a tagging file."""

from plaso.analysis import interface
from plaso.analysis import manager
from plaso.engine import tagging_file


class TaggingAnalysisPlugin(interface.AnalysisPlugin):
  """Analysis plugin that labels events according to rules in a tagging file."""

  NAME = 'tagging'

  def __init__(self):
    """Initializes a tagging analysis plugin."""
    super(TaggingAnalysisPlugin, self).__init__()
    self._tagging_rules = None

  def ExamineEvent(
      self, analysis_mediator, event, event_data, event_data_stream):
    """Labels events according to the rules in a tagging file.

    Args:
      analysis_mediator (AnalysisMediator): mediates interactions between
          analysis plugins and other components, such as storage and dfVFS.
      event (EventObject): event to examine.
      event_data (EventData): event data.
      event_data_stream (EventDataStream): event data stream.
    """
    matched_label_names = []
    for label_name, filter_objects in self._tagging_rules.items():
      for filter_object in filter_objects:
        # Note that tagging events based on existing labels is currently
        # not supported.
        if filter_object.Match(event, event_data, event_data_stream, None):
          matched_label_names.append(label_name)
          break

    if matched_label_names:
      event_tag = self._CreateEventTag(event, matched_label_names)

      analysis_mediator.ProduceEventTag(event_tag)

      for label_name in matched_label_names:
        self._analysis_counter[label_name] += 1

      self._analysis_counter['event_tags'] += 1

  def SetAndLoadTagFile(self, tagging_file_path):
    """Sets the tagging file to be used by the plugin.

    Args:
      tagging_file_path (str): path of the tagging file.
    """
    tagging_file_object = tagging_file.TaggingFile(tagging_file_path)
    self._tagging_rules = tagging_file_object.GetEventTaggingRules()


manager.AnalysisPluginManager.RegisterPlugin(TaggingAnalysisPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the SQLite-based storage."""

import os
import unittest

from plaso.containers import events
from plaso.lib import definitions
from plaso.storage.sqlite import sqlite_file

from tests import test_lib as shared_test_lib
from tests.containers import test_lib as containers_test_lib
from tests.storage import test_lib


class _TestSQLiteStorageFileV20220716(sqlite_file.SQLiteStorageFile):
  """Test class for testing format compatibility checks."""

  _FORMAT_VERSION = 20220716
  _APPEND_COMPATIBLE_FORMAT_VERSION = 20211121
  _UPGRADE_COMPATIBLE_FORMAT_VERSION = 20211121
  _READ_COMPATIBLE_FORMAT_VERSION = 20211121


class _TestSQLiteStorageFileV20221023(sqlite_file.SQLiteStorageFile):
  """Test class for testing format compatibility checks."""

  _FORMAT_VERSION = 20221023
  _APPEND_COMPATIBLE_FORMAT_VERSION = 20221023
  _UPGRADE_COMPATIBLE_FORMAT_VERSION = 20221023
  _READ_COMPATIBLE_FORMAT_VERSION = 20211121


class SQLiteStorageFileTest(test_lib.StorageTestCase):
  """Tests for the SQLite-based storage file object."""

  # pylint: disable=protected-access

  def testInitialization(self):
    """Tests the __init__ function."""
    test_store = sqlite_file.SQLiteStorageFile()
    self.assertIsNotNone(test_store)

  def testCacheAttributeContainerByIndex(self):
    """Tests the _CacheAttributeContainerByIndex function."""
    event_data_stream = events.EventDataStream()

    with shared_test_lib.TempDirectory():
      test_store = sqlite_file.SQLiteStorageFile()

      self.assertEqual(len(test_store._attribute_container_cache), 0)

      test_store._CacheAttributeContainerByIndex(event_data_stream, 0)
      self.assertEqual(len(test_store._attribute_container_cache), 1)

  def testCheckStorageMetadata(self):
    """Tests the _CheckStorageMetadata function."""
    with shared_test_lib.TempDirectory():
      test_store = sqlite_file.SQLiteStorageFile()

      metadata_values = {
          'compression_format': definitions.COMPRESSION_FORMAT_ZLIB,
          'format_version': '{0:d}'.format(test_store._FORMAT_VERSION),
          'serialization_format': definitions.SERIALIZER_FORMAT_JSON}
      test_store._CheckStorageMetadata(metadata_values)

      metadata_values['format_version'] = 'bogus'
      with self.assertRaises(IOError):
        test_store._CheckStorageMetadata(metadata_values)

      metadata_values['format_version'] = '1'
      with self.assertRaises(IOError):
        test_store._CheckStorageMetadata(metadata_values)

      metadata_values['format_version'] = '{0:d}'.format(
          test_store._FORMAT_VERSION)
      metadata_values['compression_format'] = None
      with self.assertRaises(IOError):
        test_store._CheckStorageMetadata(metadata_values)

      metadata_values['compression_format'] = (
          definitions.COMPRESSION_FORMAT_ZLIB)
      metadata_values['serialization_format'] = None
      with self.assertRaises(IOError):
        test_store._CheckStorageMetadata(metadata_values)

      metadata_values['serialization_format'] = (
          definitions.SERIALIZER_FORMAT_JSON)
      with self.assertRaises(IOError):
        test_store._CheckStorageMetadata(metadata_values)

  def testCreateAttributeContainerTable(self):
    """Tests the _CreateAttributeContainerTable function."""
    event_data_stream = events.EventDataStream()

    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      test_store = sqlite_file.SQLiteStorageFile()
      test_store.Open(path=test_path, read_only=False)

      try:
        test_store._CreateAttributeContainerTable(
            event_data_stream.CONTAINER_TYPE)

        with self.assertRaises(IOError):
          test_store._CreateAttributeContainerTable(
              event_data_stream.CONTAINER_TYPE)

      finally:
        test_store.Close()

  # TODO: add tests for _CreateAttributeContainerFromRow
  # TODO: add tests for _DeserializeAttributeContainer

  def testGetAttributeContainersWithFilter(self):
    """Tests the _GetAttributeContainersWithFilter function."""
    event_data_stream = events.EventDataStream()
    event_data_stream.md5_hash = '8f0bf95a7959baad9666b21a7feed79d'

    column_names = ['md5_hash']

    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      test_store = sqlite_file.SQLiteStorageFile()
      test_store.Open(path=test_path, read_only=False)

      try:
        containers = list(test_store._GetAttributeContainersWithFilter(
            event_data_stream.CONTAINER_TYPE, column_names=column_names))
        self.assertEqual(len(containers), 0)

        test_store.AddAttributeContainer(event_data_stream)

        containers = list(test_store._GetAttributeContainersWithFilter(
            event_data_stream.CONTAINER_TYPE, column_names=column_names))
        self.assertEqual(len(containers), 1)

        filter_expression = 'md5_hash == "8f0bf95a7959baad9666b21a7feed79d"'
        containers = list(test_store._GetAttributeContainersWithFilter(
            event_data_stream.CONTAINER_TYPE, column_names=column_names,
            filter_expression=filter_expression))
        self.assertEqual(len(containers), 1)

        filter_expression = 'md5_hash != "8f0bf95a7959baad9666b21a7feed79d"'
        containers = list(test_store._GetAttributeContainersWithFilter(
            event_data_stream.CONTAINER_TYPE, column_names=column_names,
            filter_expression=filter_expression))
        self.assertEqual(len(containers), 0)

        containers = list(test_store._GetAttributeContainersWithFilter(
            'bogus', column_names=column_names))
        self.assertEqual(len(containers), 0)

      finally:
        test_store.Close()

  def testGetCachedAttributeContainer(self):
    """Tests the _GetCachedAttributeContainer function."""
    event_data_stream = events.EventDataStream()

    with shared_test_lib.TempDirectory():
      test_store = sqlite_file.SQLiteStorageFile()

      attribute_container = test_store._GetCachedAttributeContainer(
          event_data_stream.CONTAINER_TYPE, 1)
      self.assertIsNone(attribute_container)

      test_store._CacheAttributeContainerByIndex(event_data_stream, 1)

      attribute_container = test_store._GetCachedAttributeContainer(
          event_data_stream.CONTAINER_TYPE, 1)
      self.assertIsNotNone(attribute_container)

  def testHasTable(self):
    """Tests the _HasTable function."""
    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      test_store = sqlite_file.SQLiteStorageFile()
      test_store.Open(path=test_path, read_only=False)

      try:
        test_store._CreateAttributeContainerTable(
            events.EventDataStream.CONTAINER_TYPE)

        result = test_store._HasTable(
            events.EventDataStream.CONTAINER_TYPE)
        self.assertTrue(result)

        result = test_store._HasTable('bogus')
        self.assertFalse(result)

      finally:
        test_store.Close()

  # TODO: add tests for _RaiseIfNotReadable
  # TODO: add tests for _RaiseIfNotWritable
  # TODO: add tests for _ReadAndCheckStorageMetadata
  # TODO: add tests for _SerializeAttributeContainer
  # TODO: add tests for _UpdateEventAfterDeserialize
  # TODO: add tests for _UpdateEventBeforeSerialize
  # TODO: add tests for _UpdateEventDataAfterDeserialize
  # TODO: add tests for _UpdateEventDataBeforeSerialize
  # TODO: add tests for _UpdateEventTagAfterDeserialize
  # TODO: add tests for _UpdateEventTagBeforeSerialize
  # TODO: add tests for _UpdateStorageMetadataFormatVersion

  def testWriteExistingAttributeContainer(self):
    """Tests the _WriteExistingAttributeContainer function."""
    event_data_stream = events.EventDataStream()

    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      test_store = sqlite_file.SQLiteStorageFile()
      test_store.Open(path=test_path, read_only=False)

      try:
        number_of_containers = test_store.GetNumberOfAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 0)

        test_store._WriteNewAttributeContainer(event_data_stream)

        number_of_containers = test_store.GetNumberOfAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 1)

        test_store._WriteExistingAttributeContainer(event_data_stream)

        number_of_containers = test_store.GetNumberOfAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 1)

      finally:
        test_store.Close()

  # TODO: add tests for _WriteMetadata
  # TODO: add tests for _WriteMetadataValue

  def testWriteNewAttributeContainer(self):
    """Tests the _WriteNewAttributeContainer function."""
    event_data_stream = events.EventDataStream()

    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      test_store = sqlite_file.SQLiteStorageFile()
      test_store.Open(path=test_path, read_only=False)

      try:
        number_of_containers = test_store.GetNumberOfAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 0)

        test_store._WriteNewAttributeContainer(event_data_stream)

        number_of_containers = test_store.GetNumberOfAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 1)

      finally:
        test_store.Close()

  def testAddAttributeContainer(self):
    """Tests the AddAttributeContainer function."""
    event_data_stream = events.EventDataStream()

    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      test_store = sqlite_file.SQLiteStorageFile()
      test_store.Open(path=test_path, read_only=False)

      try:
        number_of_containers = test_store.GetNumberOfAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 0)

        test_store.AddAttributeContainer(event_data_stream)

        number_of_containers = test_store.GetNumberOfAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 1)

      finally:
        test_store.Close()

      with self.assertRaises(IOError):
        test_store.AddAttributeContainer(event_data_stream)

  # TODO: add tests for CheckSupportedFormat

  def testGetAttributeContainers(self):
    """Tests the GetAttributeContainers function."""
    event_data_stream = events.EventDataStream()
    event_data_stream.md5_hash = '8f0bf95a7959baad9666b21a7feed79d'

    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      test_store = sqlite_file.SQLiteStorageFile()
      test_store.Open(path=test_path, read_only=False)

      try:
        containers = list(test_store.GetAttributeContainers(
            event_data_stream.CONTAINER_TYPE))
        self.assertEqual(len(containers), 0)

        test_store.AddAttributeContainer(event_data_stream)

        containers = list(test_store.GetAttributeContainers(
            event_data_stream.CONTAINER_TYPE))
        self.assertEqual(len(containers), 1)

        filter_expression = 'md5_hash == "8f0bf95a7959baad9666b21a7feed79d"'
        containers = list(test_store.GetAttributeContainers(
            event_data_stream.CONTAINER_TYPE,
            filter_expression=filter_expression))
        self.assertEqual(len(containers), 1)

        filter_expression = 'md5_hash != "8f0bf95a7959baad9666b21a7feed79d"'
        containers = list(test_store.GetAttributeContainers(
            event_data_stream.CONTAINER_TYPE,
            filter_expression=filter_expression))
        self.assertEqual(len(containers), 0)

        containers = list(test_store.GetAttributeContainers('bogus'))
        self.assertEqual(len(containers), 0)

      finally:
        test_store.Close()

  def testGetAttributeContainerByIdentifier(self):
    """Tests the GetAttributeContainerByIdentifier function."""
    event_data_stream = events.EventDataStream()

    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      test_store = sqlite_file.SQLiteStorageFile()
      test_store.Open(path=test_path, read_only=False)

      try:
        test_store.AddAttributeContainer(event_data_stream)

        identifier = event_data_stream.GetIdentifier()

        container = test_store.GetAttributeContainerByIdentifier(
            event_data_stream.CONTAINER_TYPE, identifier)
        self.assertIsNotNone(container)

        identifier.sequence_number = 99

        container = test_store.GetAttributeContainerByIdentifier(
            event_data_stream.CONTAINER_TYPE, identifier)
        self.assertIsNone(container)

      finally:
        test_store.Close()

  def testGetAttributeContainerByIndex(self):
    """Tests the GetAttributeContainerByIndex function."""
    event_data_stream = events.EventDataStream()

    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      test_store = sqlite_file.SQLiteStorageFile()
      test_store.Open(path=test_path, read_only=False)

      try:
        container = test_store.GetAttributeContainerByIndex(
            event_data_stream.CONTAINER_TYPE, 0)
        self.assertIsNone(container)

        test_store.AddAttributeContainer(event_data_stream)

        container = test_store.GetAttributeContainerByIndex(
            event_data_stream.CONTAINER_TYPE, 0)
        self.assertIsNotNone(container)

        container = test_store.GetAttributeContainerByIndex('bogus', 0)
        self.assertIsNone(container)

      finally:
        test_store.Close()

  def testGetNumberOfAttributeContainers(self):
    """Tests the GetNumberOfAttributeContainers function."""
    event_data_stream = events.EventDataStream()

    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      test_store = sqlite_file.SQLiteStorageFile()
      test_store.Open(path=test_path, read_only=False)

      try:
        number_of_containers = test_store.GetNumberOfAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 0)

        test_store.AddAttributeContainer(event_data_stream)

        number_of_containers = test_store.GetNumberOfAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 1)

        # Test for a supported container type that does not have a table
        # present in the storage file.
        query = 'DROP TABLE {0:s}'.format(event_data_stream.CONTAINER_TYPE)
        test_store._cursor.execute(query)
        number_of_containers = test_store.GetNumberOfAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 0)

      finally:
        test_store.Close()

  def testGetSortedEvents(self):
    """Tests the GetSortedEvents function."""
    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      test_store = sqlite_file.SQLiteStorageFile()
      test_store.Open(path=test_path, read_only=False)

      try:
        for event, event_data, event_data_stream in (
            containers_test_lib.CreateEventsFromValues(self._TEST_EVENTS)):
          test_store.AddAttributeContainer(event_data_stream)

          event_data.SetEventDataStreamIdentifier(
              event_data_stream.GetIdentifier())
          test_store.AddAttributeContainer(event_data)

          event.SetEventDataIdentifier(event_data.GetIdentifier())
          test_store.AddAttributeContainer(event)

      finally:
        test_store.Close()

      test_store = sqlite_file.SQLiteStorageFile()
      test_store.Open(path=test_path)

      try:
        test_events = list(test_store.GetSortedEvents())
        self.assertEqual(len(test_events), 4)

      finally:
        test_store.Close()

    # TODO: add test with time range.

  def testHasAttributeContainers(self):
    """Tests the HasAttributeContainers function."""
    event_data_stream = events.EventDataStream()

    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      test_store = sqlite_file.SQLiteStorageFile()
      test_store.Open(path=test_path, read_only=False)

      try:
        result = test_store.HasAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertFalse(result)

        test_store.AddAttributeContainer(event_data_stream)

        result = test_store.HasAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertTrue(result)

        result = test_store.HasAttributeContainers('bogus')
        self.assertFalse(result)

      finally:
        test_store.Close()

  # TODO: add tests for Open and Close

  def testUpdateAttributeContainer(self):
    """Tests the UpdateAttributeContainer function."""
    event_data_stream = events.EventDataStream()

    with shared_test_lib.TempDirectory() as temp_directory:
      test_path = os.path.join(temp_directory, 'plaso.sqlite')
      test_store = sqlite_file.SQLiteStorageFile()
      test_store.Open(path=test_path, read_only=False)

      try:
        number_of_containers = test_store.GetNumberOfAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 0)

        test_store.AddAttributeContainer(event_data_stream)

        number_of_containers = test_store.GetNumberOfAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 1)

        test_store.UpdateAttributeContainer(event_data_stream)

        number_of_containers = test_store.GetNumberOfAttributeContainers(
            event_data_stream.CONTAINER_TYPE)
        self.assertEqual(number_of_containers, 1)

      finally:
        test_store.Close()

  def testVersionCompatibility(self):
    """Tests the version compatibility methods."""
    with shared_test_lib.TempDirectory() as temp_directory:
      v1_storage_path = os.path.join(temp_directory, 'v20220716.sqlite')
      v1_test_store = _TestSQLiteStorageFileV20220716()
      v1_test_store.Open(path=v1_storage_path, read_only=False)
      v1_test_store.Close()

      v2_test_store_rw = _TestSQLiteStorageFileV20221023()

      with self.assertRaises((IOError, OSError)):
        v2_test_store_rw.Open(path=v1_storage_path, read_only=False)

      v2_test_store_ro = _TestSQLiteStorageFileV20221023()
      v2_test_store_ro.Open(path=v1_storage_path, read_only=True)
      v2_test_store_ro.Close()


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""File system custom event formatter helpers."""

from plaso.formatters import interface
from plaso.formatters import manager


class NTFSFileReferenceFormatterHelper(interface.CustomEventFormatterHelper):
  """NTFS file reference formatter helper."""

  IDENTIFIER = 'ntfs_file_reference'

  def FormatEventValues(self, output_mediator, event_values):
    """Formats event values using the helper.

    Args:
      output_mediator (OutputMediator): output mediator.
      event_values (dict[str, object]): event values.
    """
    file_reference = event_values.get('file_reference', None)
    if file_reference:
      event_values['file_reference'] = '{0:d}-{1:d}'.format(
          file_reference & 0xffffffffffff, file_reference >> 48)


class NTFSParentFileReferenceFormatterHelper(
    interface.CustomEventFormatterHelper):
  """NTFS parent file reference formatter helper."""

  IDENTIFIER = 'ntfs_parent_file_reference'

  def FormatEventValues(self, output_mediator, event_values):
    """Formats event values using the helper.

    Args:
      output_mediator (OutputMediator): output mediator.
      event_values (dict[str, object]): event values.
    """
    parent_file_reference = event_values.get('parent_file_reference', None)
    if parent_file_reference:
      event_values['parent_file_reference'] = '{0:d}-{1:d}'.format(
          parent_file_reference & 0xffffffffffff, parent_file_reference >> 48)


class NTFSPathHintsFormatterHelper(interface.CustomEventFormatterHelper):
  """NTFS path hints formatter helper."""

  IDENTIFIER = 'ntfs_path_hints'

  def FormatEventValues(self, output_mediator, event_values):
    """Formats event values using the helper.

    Args:
      output_mediator (OutputMediator): output mediator.
      event_values (dict[str, object]): event values.
    """
    path_hints = event_values.get('path_hints', None)
    if path_hints:
      event_values['path_hints'] = ';'.join(path_hints)


manager.FormattersManager.RegisterEventFormatterHelpers([
    NTFSFileReferenceFormatterHelper, NTFSParentFileReferenceFormatterHelper,
    NTFSPathHintsFormatterHelper])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Android Application Usage history parsers."""

import unittest

from plaso.parsers import android_app_usage

from tests.parsers import test_lib


class AndroidAppUsageParserTest(test_lib.ParserTestCase):
  """Tests for the Android Application Usage History parser."""

  def testParse(self):
    """Tests the Parse function."""
    parser = android_app_usage.AndroidAppUsageParser()
    storage_writer = self._ParseFile(['usage-history.xml'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 28)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'component': (
            'com.sec.android.widgetapp.ap.hero.accuweather.menu.MenuAdd'),
        'data_type': 'android:app_usage',
        'last_resume_time': '2013-12-09T19:28:33.047+00:00',
        'package': 'com.sec.android.widgetapp.ap.hero.accuweather'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 22)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the bloom database analysis plugin CLI arguments helper."""

import argparse
import unittest

try:
  import flor
  from plaso.analysis import bloom
  from plaso.cli.helpers import bloom_analysis
except ModuleNotFoundError:
  flor = None

from plaso.lib import errors

from tests.cli import test_lib as cli_test_lib
from tests.cli.helpers import test_lib


@unittest.skipIf(flor is None, 'missing flor support')
class BloomAnalysisArgumentsHelperTest(
    test_lib.AnalysisPluginArgumentsHelperTest):
  """Tests the bloom database analysis plugin CLI arguments helper."""

  # pylint: disable=no-member,protected-access

  _EXPECTED_OUTPUT = """\
usage: cli_helper.py [--bloom-file PATH] [--bloom-hash HASH]
                     [--bloom-label LABEL]

Test argument parser.

{0:s}:
  --bloom-file PATH, --bloom_file PATH
                        Path to the bloom database file, the default is:
                        hashlookup-full.bloom
  --bloom-hash HASH, --bloom_hash HASH
                        Type of hash to use to query the bloom database file
                        (note that hash values must be stored in upper case),
                        the default is: sha1. Supported options: md5, sha1,
                        sha256.
  --bloom-label LABEL, --bloom_label LABEL
                        Label to apply to events, the default is:
                        bloom_present.
""".format(cli_test_lib.ARGPARSE_OPTIONS)

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = argparse.ArgumentParser(
        prog='cli_helper.py',
        description='Test argument parser.', add_help=False,
        formatter_class=cli_test_lib.SortedArgumentsHelpFormatter)

    bloom_analysis.BloomAnalysisArgumentsHelper.AddArguments(
        argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()
    analysis_plugin = bloom.BloomAnalysisPlugin()

    with self.assertRaises(errors.BadConfigOption):
      bloom_analysis.BloomAnalysisArgumentsHelper.ParseOptions(
          options, analysis_plugin)

    with self.assertRaises(errors.BadConfigObject):
      bloom_analysis.BloomAnalysisArgumentsHelper.ParseOptions(
          options, None)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the selinux log file text parser plugin."""

import unittest

from plaso.parsers.text_plugins import selinux

from tests.parsers.text_plugins import test_lib


class SELinuxTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the selinux log file text parser plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = selinux.SELinuxTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['selinux.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 7)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 4)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Test case: normal entry.
    expected_event_values = {
        'audit_type': 'LOGIN',
        'body': (
            'pid=25443 uid=0 old auid=4294967295 new auid=0 old ses=4294967295 '
            'new ses=1165'),
        'data_type': 'selinux:line',
        'last_written_time': '2012-05-24T07:40:01.174+00:00',
        'pid': '25443'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

    # Test case: short date.
    expected_event_values = {
        'audit_type': 'SHORTDATE',
        'body': 'check rounding',
        'data_type': 'selinux:line',
        'last_written_time': '2012-05-24T07:40:01.000+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

    # Test case: no message.
    expected_event_values = {
        'audit_type': 'NOMSG',
        'data_type': 'selinux:line',
        'last_written_time': '2012-05-24T07:40:22.174+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)

    # Test case: under score.
    expected_event_values = {
        'audit_type': 'UNDER_SCORE',
        'body': (
            'pid=25444 uid=0 old auid=4294967295 new auid=54321 old '
            'ses=4294967295 new ses=1166'),
        'data_type': 'selinux:line',
        'last_written_time': '2012-05-24T07:47:46.174+00:00',
        'pid': '25444'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 3)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Text parser plugin for PostgreSQL application log files.

This is a multi-line log format that records internal database application
logs as well as authentication attempts.

Also see:
  https://www.postgresql.org/docs/current/runtime-config-logging.html
"""

import pyparsing

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.lib import errors
from plaso.parsers import text_parser
from plaso.parsers.text_plugins import interface


class PostgreSQLEventData(events.EventData):
  """PostgreSQL application log data.

  Attributes:
    log_line (str): log message.
    pid (int): process identifier (PID).
    recorded_time (dfdatetime.DateTimeValues): date and time the log entry
        was recorded.
    severity (str): severity.
    user (str): "user@database" string if present. Records the user account and
        database name that was authenticated or attempting to authenticate.
  """

  DATA_TYPE = 'postgresql:application_log:entry'

  def __init__(self):
    """Initializes event data."""
    super(PostgreSQLEventData, self).__init__(data_type=self.DATA_TYPE)
    self.log_line = None
    self.pid = None
    self.recorded_time = None
    self.severity = None
    self.user = None


class PostgreSQLTextPlugin(interface.TextPlugin):
  """Text parser plugin for PostgreSQL application log files."""

  NAME = 'postgresql'
  DATA_FORMAT = 'PostgreSQL application log file'

  ENCODING = 'utf-8'

  _INTEGER = pyparsing.Word(pyparsing.nums).setParseAction(
      lambda tokens: int(tokens[0], 10))

  _TWO_DIGITS = pyparsing.Word(pyparsing.nums, exact=2).setParseAction(
      lambda tokens: int(tokens[0], 10))

  _THREE_DIGITS = pyparsing.Word(pyparsing.nums, exact=3).setParseAction(
      lambda tokens: int(tokens[0], 10))

  _FOUR_DIGITS = pyparsing.Word(pyparsing.nums, exact=4).setParseAction(
      lambda tokens: int(tokens[0], 10))

  # Date and time values are formatted as: YYYY-MM-DD hh:mm:ss.### UTC
  # For example: 2022-04-12 00:16:05.526 UTC
  _DATE_TIME = (
      pyparsing.LineStart() +
      _FOUR_DIGITS.setResultsName('year') + pyparsing.Suppress('-') +
      _TWO_DIGITS.setResultsName('month') + pyparsing.Suppress('-') +
      _TWO_DIGITS.setResultsName('day_of_month') +
      _TWO_DIGITS.setResultsName('hours') + pyparsing.Suppress(':') +
      _TWO_DIGITS.setResultsName('minutes') + pyparsing.Suppress(':') +
      _TWO_DIGITS.setResultsName('seconds') +
      pyparsing.Optional(
          pyparsing.Suppress('.') +
          _THREE_DIGITS.setResultsName('milliseconds'))).setResultsName(
              'date_time')

  _TIME_ZONE = pyparsing.Word(pyparsing.printables).setResultsName('time_zone')

  _PID = (
      pyparsing.Suppress('[') + pyparsing.OneOrMore(_INTEGER) +
      pyparsing.Optional(pyparsing.Literal('-')) +
      pyparsing.ZeroOrMore(_INTEGER) +
      pyparsing.Suppress(']')).setResultsName('pid')

  _USER_AND_DATABASE = (
      pyparsing.Word(pyparsing.alphanums) +
      pyparsing.Literal('@') +
      pyparsing.Word(pyparsing.alphanums)).setResultsName('user_and_database')

  _SEVERITY = pyparsing.Word(pyparsing.string.ascii_uppercase)

  _LOG_LINE_END = pyparsing.StringEnd() | (_DATE_TIME + _TIME_ZONE)

  _END_OF_LINE = pyparsing.Suppress(pyparsing.LineEnd())

  _LOG_LINE = (
      _DATE_TIME + _TIME_ZONE + _PID + pyparsing.Optional(_USER_AND_DATABASE) +
      _SEVERITY.setResultsName('severity') +
      pyparsing.Suppress(':') +
      pyparsing.SkipTo(_LOG_LINE_END).setResultsName('log_line') +
      pyparsing.ZeroOrMore(_END_OF_LINE))

  _LINE_STRUCTURES = [('log_line', _LOG_LINE)]

  VERIFICATION_GRAMMAR = _LOG_LINE

  # TODO: move this into timeliner

  # Extracted from /usr/share/postgresql/13/timezonesets/Default
  # See https://www.postgresql.org/docs/current/datetime-config-files.html
  _PSQL_TIME_ZONE_MAPPING = {
        'ACDT': 'Australia/Adelaide',
        'ACST': 'Australia/Adelaide',
        'ADT': 'America/Glace_Bay',
        'AEDT': 'Australia/Brisbane',
        'AEST': 'Australia/Brisbane',
        'AKDT': 'America/Anchorage',
        'AKST': 'America/Anchorage',
        'AST': 'America/Anguilla',
        'AWST': 'Australia/Perth',
        'BST': 'Europe/London',
        'CDT': 'America/Chicago',
        'CEST': 'Africa/Ceuta',
        'CET': 'Africa/Algiers',
        'CETDST': 'Africa/Ceuta',
        'CST': 'America/Chicago',
        'EAT': 'Africa/Addis_Ababa',
        'EDT': 'America/Detroit',
        'EEST': 'Africa/Cairo',
        'EET': 'Africa/Cairo',
        'EETDST': 'Africa/Cairo',
        'EST': 'America/Cancun',
        'GMT': 'Africa/Abidjan',
        'HKT': 'Asia/Hong_Kong',
        'HST': 'Pacific/Honolulu',
        'IDT': 'Asia/Jerusalem',
        'IST': 'Asia/Jerusalem',
        'JST': 'Asia/Tokyo',
        'KST': 'Asia/Seoul',
        'MDT': 'America/Boise',
        'MSK': 'Europe/Moscow',
        'MST': 'America/Boise',
        'NDT': 'America/St_Johns',
        'NST': 'America/St_Johns',
        'NZDT': 'Antarctica/McMurdo',
        'NZST': 'Antarctica/McMurdo',
        'PDT': 'America/Dawson',
        'PKST': 'Asia/Karachi',
        'PKT': 'Asia/Karachi',
        'PST': 'America/Dawson',
        'SAST': 'Africa/Johannesburg',
        'UCT': 'Etc/UCT',
        'WAT': 'Africa/Bangui',
        'WET': 'Africa/Casablanca',
        'WETDST': 'Atlantic/Canary'}

  def _ParseRecord(self, parser_mediator, key, structure):
    """Parses a pyparsing structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: if the structure cannot be parsed.
    """
    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    log_line = self._GetValueFromStructure(
        structure, 'log_line', default_value='')
    log_line = log_line.lstrip().rstrip()

    pids = self._GetValueFromStructure(structure, 'pid', default_value=[])

    time_zone_string = self._GetValueFromStructure(structure, 'time_zone')

    user_and_database = self._GetValueFromStructure(
        structure, 'user_and_database', default_value='')
    user_and_database = ''.join(user_and_database)

    # TODO: move this into timeliner
    time_zone_string = self._PSQL_TIME_ZONE_MAPPING.get(
        time_zone_string, time_zone_string)

    date_time = self._ParseTimeElements(time_elements_structure)
    if time_zone_string != 'UTC':
      date_time.is_local_time = True
      date_time.time_zone_hint = time_zone_string

    event_data = PostgreSQLEventData()
    event_data.log_line = log_line or None
    event_data.pid = ''.join([str(pid) for pid in pids])
    event_data.recorded_time = date_time
    event_data.severity = self._GetValueFromStructure(structure, 'severity')
    event_data.user = user_and_database or None

    parser_mediator.ProduceEventData(event_data)

  def _ParseTimeElements(self, time_elements_structure):
    """Parses date and time elements of a log line.

    Args:
      time_elements_structure (pyparsing.ParseResults): date and time elements
          of a log line.

    Returns:
      dfdatetime.TimeElements: date and time value.

    Raises:
      ParseError: if a valid date and time value cannot be derived from
          the time elements.
    """
    # Ensure time_elements_tuple is not a pyparsing.ParseResults otherwise
    # copy.deepcopy() of the dfDateTime object will fail on Python 3.8 with:
    # "TypeError: 'str' object is not callable" due to pyparsing.ParseResults
    # overriding __getattr__ with a function that returns an empty string when
    # named token does not exist.
    try:
      if len(time_elements_structure) == 6:
        year, month, day_of_month, hours, minutes, seconds = (
            time_elements_structure)

        date_time = dfdatetime_time_elements.TimeElements(time_elements_tuple=(
            year, month, day_of_month, hours, minutes, seconds))

      else:
        year, month, day_of_month, hours, minutes, seconds, milliseconds = (
            time_elements_structure)

        date_time = dfdatetime_time_elements.TimeElementsInMilliseconds(
            time_elements_tuple=(
                year, month, day_of_month, hours, minutes, seconds,
                milliseconds))

      return date_time

    except (TypeError, ValueError) as exception:
      raise errors.ParseError(
          'Unable to parse time elements with error: {0!s}'.format(exception))

  def CheckRequiredFormat(self, parser_mediator, text_reader):
    """Check if the log record has the minimal structure required by the parser.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    try:
      structure = self._VerifyString(text_reader.lines)
    except errors.ParseError:
      return False

    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    try:
      self._ParseTimeElements(time_elements_structure)
    except errors.ParseError:
      return False

    return True


text_parser.TextLogParser.RegisterPlugin(PostgreSQLTextPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the operating system independent (generic) preprocessor plugins."""

import unittest

from dfvfs.helpers import fake_file_system_builder
from dfvfs.helpers import file_system_searcher
from dfvfs.path import fake_path_spec

from plaso.preprocessors import generic
from plaso.preprocessors import mediator
from plaso.storage.fake import writer as fake_writer

from tests import test_lib as shared_test_lib
from tests.preprocessors import test_lib


class DetermineOperatingSystemPluginTest(
    test_lib.ArtifactPreprocessorPluginTestCase):
  """Tests for the plugin to determine the operating system."""

  def testCollect(self):
    """Tests the Collect function."""
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    test_file_path = shared_test_lib.GetTestFilePath(['SOFTWARE'])
    file_system_builder.AddFileReadData(
        '/Windows/System32/config/SOFTWARE', test_file_path)
    test_file_path = shared_test_lib.GetTestFilePath(['SYSTEM'])
    file_system_builder.AddFileReadData(
        '/Windows/System32/config/SYSTEM', test_file_path)

    storage_writer = fake_writer.FakeStorageWriter()
    test_mediator = mediator.PreprocessMediator(storage_writer)

    mount_point = fake_path_spec.FakePathSpec(location='/')
    searcher = file_system_searcher.FileSystemSearcher(
        file_system_builder.file_system, mount_point)

    plugin = generic.DetermineOperatingSystemPlugin()

    storage_writer.Open()

    try:
      plugin.Collect(
          test_mediator, None, searcher, file_system_builder.file_system)
    finally:
      storage_writer.Close()

    operating_system = test_mediator.GetValue('operating_system')
    self.assertEqual(operating_system, 'Windows NT')


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for Windows Defender scan DetectionHistory files."""

import os

from dfdatetime import filetime as dfdatetime_filetime

from dtfabric.runtime import data_maps as dtfabric_data_maps

from plaso.containers import events
from plaso.lib import dtfabric_helper
from plaso.lib import specification
from plaso.parsers import interface
from plaso.parsers import manager


class WindowsDefenderHistoryEventData(events.EventData):
  """Windows Defender scan DetectionHistory event data.

  Attributes:
    additional_filenames (list[str]): locations of additional detected files.
    container_filenames (list[str]): location of files detected inside a
        container.
    filename (str): name of the file that the threat was detected in.
    host_and_user (str): name of the host and user in "DOMAIN\\USER" format.
    process (str): name of the process that caused the detection.
    recorded_time (dfdatetime.DateTimeValues): date and time the log entry
        was recorded.
    sha256 (str): SHA-256 hash of the file.
    threat_name (str): name of the threat that was detected.
    web_filenames (list[str]): URI of files detected as downloaded from the web.
  """

  DATA_TYPE = 'av:defender:detection_history'

  def __init__(self):
    """Initializes event data."""
    super(WindowsDefenderHistoryEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.additional_filenames = None
    self.container_filenames = None
    self.filename = None
    self.host_and_user = None
    self.recorded_time = None
    self.process = None
    self.sha256 = None
    self.threat_name = None
    self.web_filenames = None


class WinDefenderHistoryParser(
    interface.FileObjectParser, dtfabric_helper.DtFabricHelper):
  """Parses a Windows Defender scan DetectionHistory file."""

  NAME = 'windefender_history'
  DATA_FORMAT = 'Windows Defender scan DetectionHistory file'

  _FILE_SIGNATURE = 'Magic.Version:1.2'

  _VALUE_DESCRIPTIONS = [
      {0: 'Threat identifier', 1: 'Identifier'},
      {0: 'UnknownMagic1', 1: 'Threat name', 4: 'Category'},
      {0: 'UnknownMagic2',
       1: 'Resource type',
       2: 'Resource location',
       4: 'Threat tracking data size',
       5: 'Threat tracking data',
       6: 'Last threat status change time',
       12: 'Domain user1',
       14: 'Process name',
       18: 'Initial detection time',
       20: 'Remediation time',
       24: 'Domain user2'}]

  _DEFINITION_FILE = os.path.join(
      os.path.dirname(__file__), 'windefender_history.yaml')

  _VALUE_DATA_TYPE_BINARY_DATA = 0x00000028
  _VALUE_DATA_TYPE_FILETIME = 0x0000000a
  _VALUE_DATA_TYPE_GUID = 0x0000001e
  _VALUE_DATA_TYPE_STRING = 0x00000015

  _VALUE_DATA_TYPES_INTEGER = frozenset([
      0x00000000, 0x00000005, 0x00000006, 0x00000008])

  @classmethod
  def GetFormatSpecification(cls):
    """Retrieves the format specification.

    Returns:
      FormatSpecification: format specification.
    """
    format_specification = specification.FormatSpecification(cls.NAME)
    format_specification.AddNewSignature(
        cls._FILE_SIGNATURE.encode('utf-16-le'), offset=0x30)
    return format_specification

  def _ReadThreatTrackingData(self, threat_tracking_data, file_offset):
    """Reads the threat tracking data.

    Args:
      threat_tracking_data (bytes): threat tracking data.
      file_offset (int): offset of the threat tracking data relative to
          the start of the file.

    Returns:
      dict[str, str]: Mapping of threat tracking keys to values.

    Raises:
      IOError: if the threat tracking data cannot be read.
    """
    threat_tracking = {}
    data_type_map = self._GetDataTypeMap('uint32le')

    values_data_size = self._ReadStructureFromByteStream(
        threat_tracking_data, 0, data_type_map)

    if values_data_size != 1:
      values_data_offset = 4
      values_data_end_offset = values_data_size
    else:
      header = self._ReadThreatTrackingHeader(threat_tracking_data)

    values_data_offset = header.header_size + 4
    values_data_end_offset = header.total_data_size

    while values_data_offset < values_data_end_offset:
      threat_value, data_size = self._ReadThreatTrackingValue(
          threat_tracking_data[values_data_offset:],
          file_offset + values_data_offset)
      if hasattr(threat_value, 'value_string'):
        threat_tracking[threat_value.key_string] = threat_value.value_string
      if hasattr(threat_value, 'value_integer'):
        threat_tracking[threat_value.key_string] = threat_value.value_integer
      values_data_offset += data_size

    return threat_tracking

  def _ReadThreatTrackingHeader(self, threat_tracking_data):
    """Reads the threat tracking header.

    Args:
      threat_tracking_data (bytes): threat tracking data.

    Returns:
      threat_tracking_header: threat tracking header.

    Raises:
      IOError: if the threat tracking header cannot be read.
    """
    data_type_map = self._GetDataTypeMap('threat_tracking_header')

    return self._ReadStructureFromByteStream(
        threat_tracking_data, 0, data_type_map)

  def _ReadThreatTrackingValue(self, threat_tracking_data, file_offset):
    """Reads the threat tracking value.

    Args:
      threat_tracking_data (bytes): threat tracking data.
      file_offset (int): offset of the threat tracking data relative to
          the start of the file.

    Returns:
      tuple[threat_tracking_value, int]: threat tracking value and
          data size.

    Raises:
      IOError: if the threat tracking value cannot be read.
    """
    data_type_map = self._GetDataTypeMap('threat_tracking_value')

    context = dtfabric_data_maps.DataTypeMapContext()

    threat_tracking_value = self._ReadStructureFromByteStream(
        threat_tracking_data, file_offset, data_type_map, context=context)

    return threat_tracking_value, context.byte_size

  def _ReadValue(self, file_object, file_offset, parser_mediator):
    """Reads the value.

    Args:
      file_object (file): file-like object.
      file_offset (int): offset of the value relative to the start of the file.
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.

    Returns:
      object: value.

    Raises:
      IOError: if the value cannot be read.
      ParseError: when the value data type is unknown.
    """
    data_type_map = self._GetDataTypeMap('detection_history_value')

    value, _ = self._ReadStructureFromFileObject(
        file_object, file_offset, data_type_map)

    value_object = None
    if value.data_type in self._VALUE_DATA_TYPES_INTEGER:
      value_object = value.value_integer
    elif value.data_type == self._VALUE_DATA_TYPE_FILETIME:
      value_object = value.value_filetime
    elif value.data_type == self._VALUE_DATA_TYPE_STRING:
      value_object = value.value_string
    elif value.data_type == self._VALUE_DATA_TYPE_GUID:
      value_object = value.value_guid
    elif value.data_type == self._VALUE_DATA_TYPE_BINARY_DATA:
      value_object = value.data
    else:
      parser_mediator.ProduceExtractionWarning(
          'unknown value data type: {0!s}'.format(value.data_type))
      value_object = value.data

    return value_object

  def ParseFileObject(self, parser_mediator, file_object):
    """Parses a Windows Defender History file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between
            parsers and other components, such as storage and dfVFS.
      file_object (dfvfs.FileIO): file-like object.

    Raises:
      WrongParser: when the file cannot be parsed.
    """
    value_tuples = []

    file_offset = 0
    file_size = file_object.get_size()

    while file_offset < file_size:
      value_object = self._ReadValue(file_object, file_offset, parser_mediator)
      value_tuples.append((file_offset, value_object))
      file_offset = file_object.tell()

    resource_name = ''
    threat_attributes = {'Resources': []}
    value_index_set = 0
    value_index = 0

    for file_offset, value_object in value_tuples:
      if value_object == self._FILE_SIGNATURE:
        if value_index_set < 2:
          value_index_set += 1
        value_index = 0

      if (value_index_set, value_index) == (2, 5):
        threat_attributes.update(
            self._ReadThreatTrackingData(value_object, file_offset + 8))

      else:
        description = self._VALUE_DESCRIPTIONS[value_index_set].get(
            value_index, 'UNKNOWN_{0:d}_{1:d}'.format(
                value_index_set, value_index))

        value_string = '{0!s}'.format(value_object)

        if description == 'Resource type':
          resource_name = value_string

        elif description == 'Resource location' and resource_name is not None:
          threat_attributes['Resources'].append(
              {'Type': resource_name, 'Location': value_string})
          resource_name = None

        else:
          threat_attributes[description] = value_string

      value_index += 1

    filenames = [
        threat_attribute['Location']
        for threat_attribute in threat_attributes['Resources']
        if threat_attribute['Type'] == 'file']

    if not filenames:
      filename = threat_attributes.get('CONTEXT_DATA_FILENAME', None)
      process_ppid = threat_attributes.get('CONTEXT_DATA_PROCESS_PPID', None)

      filenames = [','.join(list(filter(None, [filename, process_ppid])))]

    web_files = [
        threat_attribute['Location']
        for threat_attribute in threat_attributes['Resources']
        if threat_attribute['Type'] == 'webfile']

    container_files = [
        threat_attribute['Location']
        for threat_attribute in threat_attributes['Resources']
        if threat_attribute['Type'] == 'containerfile']

    additional_filenames = [
        threat_attribute['Location']
        for threat_attribute in threat_attributes['Resources']
        if 'file' not in threat_attribute['Type']]

    timestamp = threat_attributes.get('ThreatTrackingStartTime', 0)

    event_data = WindowsDefenderHistoryEventData()
    event_data.additional_filenames = additional_filenames
    event_data.container_filenames = container_files
    event_data.filename = filenames[0]
    event_data.host_and_user = threat_attributes.get('Domain user1', None)
    event_data.process = threat_attributes.get('Process name', None)
    event_data.recorded_time = dfdatetime_filetime.Filetime(
        timestamp=timestamp)
    event_data.sha256 = threat_attributes.get('ThreatTrackingSha256', None)
    event_data.threat_name = threat_attributes.get('Threat name', None)
    event_data.web_filenames = web_files

    parser_mediator.ProduceEventData(event_data)


manager.ParsersManager.RegisterParser(WinDefenderHistoryParser)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Safari cookie parser."""

import unittest

from plaso.parsers import safari_cookies

from tests.parsers import test_lib


class SafariCookieParserTest(test_lib.ParserTestCase):
  """Tests for the Safari cookie parser."""

  def testParseFile(self):
    """Tests the Parse function on a Safari binary cookies file."""
    parser = safari_cookies.BinaryCookieParser()
    storage_writer = self._ParseFile(['Cookies.binarycookies'], parser)

    # There should be:
    # * 207 events in total
    # * 182 events from the safari cookie parser
    # * 25 event from the cookie plugins

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 106)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'cookie_name': 'fpc',
        'cookie_value': (
            'd=0dTg3Ou32s3MrAJ2iHjFph100Tw3E1HTfDOTly0GfJ2g4W.mXpy54F9fjBFfXMw'
            '4YyWAG2cT2FVSqOvGGi_Y1OPrngmNvpKPPyz5gIUP6x_EQeM7bR3jsrg_F1UXVOgu'
            '6JgkFwqO5uHrv4HiL05qb.85Bl.V__HZI5wpAGOGPz1XHhY5mOMH.g.pkVDLli36W'
            '2iuYwA-&v=2'),
        'data_type': 'safari:cookie:entry',
        'expiration_time': '2014-07-08T05:24:29.000000+00:00',
        'creation_time': '2013-07-08T17:24:30.000000+00:00',
        'path': '/',
        'url': '.www.yahoo.com'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 26)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()
